@@ -1271,9 +1271,19 @@ load_files(Files, Options) :-
 	'$ifcompiling'('$qlf_start_module'(Module)),
 	'$export_ops'(Ops, Module, File),
 	'$consult_stream'(In, File),
-	Module:'$check_export',
+	'$check_export'(Module),
 	'$ifcompiling'('$qlf_end_part').
 
+'$check_export'(Module) :-
+	'$undefined_export'(Module, UndefList),
+	(   '$member'(Undef, UndefList),
+	    strip_module(Undef, _, Local),
+	    print_message(error,
+			  undefined_export(Module, Local)),
+	    fail
+	;   true
+	).
+
 
 %%	'$import_list'(+TargetModule, +FromModule, +Import, +Reexport) is det.
 %
