@@ -30,9 +30,10 @@
 */
 
 :- module(settings,
-	  [ setting/4,			% :Name, +Type, +Default, +Comment, +Src
+	  [ setting/4,			% :Name, +Type, +Default, +Comment
 	    setting/2,			% :Name, ?Value
 	    set_setting/2,		% :Name, +Value
+	    set_setting_default/2,	% :Name, +Value
 	    restore_setting/1,		% :Name
 	    load_settings/1,		% +File
 	    load_settings/2,		% +File, +Options
@@ -78,7 +79,8 @@ access, loading and saving of settings.
 */
 
 :- dynamic
-	value/3,			% Name, Module, Value
+	st_value/3,			% Name, Module, Value
+	st_default/3,			% Name, Module, Value
 	local_file/1.			% Path
 
 :- multifile
@@ -88,9 +90,17 @@ access, loading and saving of settings.
 	setting(:, +, +, +),
 	setting(:, ?),
 	set_setting(:, +),
+	set_setting_default(:, +),
 	current_setting(:),
 	restore_setting(:).
 
+curr_setting(Name, Module, Type, Default, Comment) :-
+	current_setting(Name, Module, Type, Default0, Comment, _Src),
+	(   st_default(Name, Module, Default1)
+	->  Default = Default1
+	;   Default = Default0
+	).
+
 %%	setting(Name, Type, Default, Comment) is det.
 %
 %	Define a setting. Name denotes the name of the setting, Type its
@@ -122,7 +132,7 @@ user:term_expansion((:- setting(QName, Type, Default, Comment)),
 	(   current_setting(Name, Module, _, _, _, OldLoc)
 	->  format(string(Message),
 		   'Already defined at: ~w', [OldLoc]),
-	    throw(error(permission_error(redefine, setting, Name),
+	    throw(error(permission_error(redefine, setting, Module:Name),
 			context(Message, _)))
 	;   source_location(File, Line)
 	->  Expanded = settings:current_setting(Name, Module, Type, Default,
@@ -143,9 +153,9 @@ to_atom(String, Atom) :-
 setting(QName, Value) :-
 	strip_module(QName, Module, Name),
 	(   ground(Name)
-	->  (   value(Name, Module, Value0)
+	->  (   st_value(Name, Module, Value0)
 	    ->  Value = Value0
-	    ;   current_setting(Name, Module, Type, Default, _, _)
+	    ;   curr_setting(Name, Module, Type, Default, _)
 	    ->	eval_default(Default, Module, Type, Value)
 	    ;	existence_error(setting, Module:Name)
 	    )
@@ -154,6 +164,18 @@ setting(QName, Value) :-
 	).
 
 
+:- dynamic
+	setting_cache/3.
+:- volatile
+	setting_cache/3.
+
+%%	clear_setting_cache is det.
+%
+%	Clear the cache for evaluation of default values.
+
+clear_setting_cache :-
+	retractall(setting_cache(_,_,_)).
+
 %%	eval_default(+Default, +Module, +Type, -Value) is det.
 %
 %	Convert the settings default value. The notation allows for some
@@ -178,11 +200,6 @@ setting(QName, Value) :-
 %		If Type is =atom=, concatenate A+B+....  Elements of the
 %		expression can be env(Name).
 
-:- dynamic
-	setting_cache/3.
-:- volatile
-	setting_cache/3.
-
 :- multifile
 	eval_default/3.			% +Default, +Type, -Value
 
@@ -319,26 +336,27 @@ numeric_type(between(L,_), Type) :-
 set_setting(QName, Value) :-
 	strip_module(QName, Module, Name),
 	must_be(atom, Name),
-	(   current_setting(Name, Module, Type, Default0, _Comment, _Src),
+	(   curr_setting(Name, Module, Type, Default0, _Comment),
 	    eval_default(Default0, Module, Type, Default)
 	->  (   Value == Default
 	    ->	retract_setting(Module:Name)
-	    ;	value(Name, Module, Value)
+	    ;	st_value(Name, Module, Value)
 	    ->	true
 	    ;	check_type(Type, Value)
 	    ->	setting(Module:Name, Old),
 	        retract_setting(Module:Name),
 	        assert_setting(Module:Name, Value),
-		broadcast(settings(changed(Module:Name, Old, Value)))
+		broadcast(settings(changed(Module:Name, Old, Value))),
+		clear_setting_cache	% might influence dependent settings.
 	    )
 	;   existence_error(setting, Name)
 	).
 
 retract_setting(Module:Name) :-
-	retractall(value(Name, Module, _)).
+	retractall(st_value(Name, Module, _)).
 
 assert_setting(Module:Name, Value) :-
-	assert(value(Name, Module, Value)).
+	assert(st_value(Name, Module, Value)).
 
 %%	restore_setting(:Name) is det.
 %
@@ -349,7 +367,7 @@ assert_setting(Module:Name, Value) :-
 restore_setting(QName) :-
 	strip_module(QName, Module, Name),
 	must_be(atom, Name),
-	(   value(Name, Module, Old)
+	(   st_value(Name, Module, Old)
 	->  retract_setting(Module:Name),
 	    setting(Module:Name, Value),
 	    (	Old \== Value
@@ -357,6 +375,28 @@ restore_setting(QName) :-
 	    ;	true
 	    )
 	;   true
+	).
+
+%%	set_setting_default(:Name, +Default) is det.
+%
+%	Change the default for a setting.  The   effect  is  the same as
+%	set_setting/2, but the new value is  considered the default when
+%	saving and restoring  a  setting.  It   is  intended  to  change
+%	application defaults in a particular context.
+
+set_setting_default(QName, Default) :-
+	strip_module(QName, Module, Name),
+	must_be(atom, Name),
+	(   current_setting(Name, Module, Type, Default0, _Comment, _Src)
+	->  retractall(settings:st_default(Name, Module, _)),
+	    retract_setting(Module:Name),
+	    (   Default == Default0
+	    ->	true
+	    ;	assert(settings:st_default(Name, Module, Default))
+	    ),
+	    eval_default(Default, Module, Type, Value),
+	    set_setting(Module:Name, Value)
+	;   existence_error(setting, Module:Name)
 	).
 
 
@@ -400,7 +440,8 @@ load_settings(File, Options) :-
 	assert(local_file(Path)),
 	open(Path, read, In, [encoding(utf8)]),
 	read_setting(In, T0),
-	call_cleanup(load_settings(T0, In, Options), close(In)).
+	call_cleanup(load_settings(T0, In, Options), close(In)),
+	clear_setting_cache.
 load_settings(File, _) :-
 	absolute_file_name(File, Path,
 			   [ access(write),
@@ -426,19 +467,19 @@ read_setting(In, Term) :-
 %	Store setting loaded from file in the Prolog database.
 
 store_setting(setting(Module:Name, Value), _) :-
-	current_setting(Name, Module, Type, Default0, _Commentm, _Src), !,
+	curr_setting(Name, Module, Type, Default0, _Commentm), !,
 	eval_default(Default0, Module, Type, Default),
 	(   Value == Default
 	->  true
 	;   check_type(Type, Value)
-	->  retractall(value(Name, Module, _)),
-	    assert(value(Name, Module, Value)),
+	->  retractall(st_value(Name, Module, _)),
+	    assert(st_value(Name, Module, Value)),
 	    broadcast(settings(changed(Module:Name, Default, Value)))
 	).
 store_setting(setting(Module:Name, Value), Options) :- !,
 	(   option(undefined(load), Options, load)
-	->  retractall(value(Name, Module, _)),
-	    assert(value(Name, Module, Value))
+	->  retractall(st_value(Name, Module, _)),
+	    assert(st_value(Name, Module, Value))
 	;   existence_error(setting, Module:Name)
 	).
 store_setting(Term, _) :-
@@ -475,8 +516,8 @@ write_setting_header(Out) :-
 	format(Out, '*/~n~n', []).
 
 save_setting(Out, Module:Name) :-
-	current_setting(Name, Module, Type, Default, Comment, _Src),
-	(   value(Name, Module, Value),
+	curr_setting(Name, Module, Type, Default, Comment),
+	(   st_value(Name, Module, Value),
 	    \+ ( eval_default(Default, Module, Type, DefValue),
 		 debug(setting, '~w <-> ~w~n', [DefValue, Value]),
 	         DefValue =@= Value
@@ -512,11 +553,11 @@ current_setting(Module:Name) :-
 setting_property(Setting, Property) :-
 	ground(Setting), !,
 	Setting = Module:Name,
-	current_setting(Name, Module, Type, Default, Comment, _Src), !,
+	curr_setting(Name, Module, Type, Default, Comment), !,
 	setting_property(Property, Module, Type, Default, Comment).
 setting_property(Setting, Property) :-
 	Setting = Module:Name,
-	current_setting(Name, Module, Type, Default, Comment, _Src),
+	curr_setting(Name, Module, Type, Default, Comment),
 	setting_property(Property, Module, Type, Default, Comment).
 
 setting_property(type(Type),       _, Type, _,        _).
@@ -536,7 +577,7 @@ list_settings :-
 	       list_setting(Setting)).
 
 list_setting(Module:Name) :-
-	current_setting(Name, Module, Type, Default0, Comment, _Src),
+	curr_setting(Name, Module, Type, Default0, Comment),
 	eval_default(Default0, Module, Type, Default),
 	setting(Module:Name, Value),
 	(   Value \== Default
