@@ -125,9 +125,12 @@
 :- use_module(library(lists),[member/2,append/3,permutation/2,reverse/2]).
 :- use_module(library(ordsets)).
 %% SWI end
-:- use_module(hprolog).
+:- use_module(library(dialect/hprolog)).
 :- use_module(pairlist).
 :- include(chr_op).
+:- if(current_prolog_flag(dialect, swi)).
+:- use_module(library(dialect/yap), [atomic_concat/3]).
+:- endif.
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%
@@ -2463,22 +2466,6 @@ atom_concat_list([X|Xs],A) :-
 	atom_concat_list(Xs,B),
 	atomic_concat(X,B,A).
 
-atomic_concat(A,B,C) :-
-	make_atom(A,AA),
-	make_atom(B,BB),
-	atom_concat(AA,BB,C).
-
-make_atom(A,AA) :-
-	(
-	  atom(A) ->
-	  AA = A
-	;
-	  number(A) ->
-	  number_codes(A,AL),
-	  atom_codes(AA,AL)
-	).
-
-
 set_elems([],_).
 set_elems([X|Xs],X) :-
 	set_elems(Xs,X).
