@@ -1667,7 +1667,7 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
     { int val = va_arg(args, int);
 
       if( setsockopt(s->socket, SOL_SOCKET, SO_REUSEADDR,
-		     (const char *)&val, sizeof(val)) == -1)
+		     (const char *)&val, sizeof(val)) == -1 )
       { nbio_error(h_errno, TCP_HERRNO);
 	rc = -1;
       } else
@@ -1675,11 +1675,31 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
 
       break;
     }
+    case TCP_NO_DELAY:
+#ifdef TCP_NODELAY
+    { int val = va_arg(args, int);
+
+#ifndef IPPROTO_TCP			/* Is this correct? */
+#define IPPROTO_TCP SOL_SOCKET
+#endif
+      if ( setsockopt(s->socket, IPPROTO_TCP, TCP_NODELAY,
+		      (const char *)&val, sizeof(val)) == -1 )
+      { nbio_error(h_errno, TCP_HERRNO);
+	rc = -1;
+      } else
+      { rc = 0;
+      }
+
+      break;
+    }
+#else
+    rc = -2;				/* not implemented */
+#endif
     case UDP_BROADCAST:
     { int val = va_arg(args, int);
 
-      if( setsockopt(s->socket, SOL_SOCKET, SO_BROADCAST,
-		     (const char *)&val, sizeof(val)) == -1)
+      if ( setsockopt(s->socket, SOL_SOCKET, SO_BROADCAST,
+		     (const char *)&val, sizeof(val)) == -1 )
       { nbio_error(h_errno, TCP_HERRNO);
 	rc = -1;
       } else
@@ -2172,7 +2192,6 @@ nbio_close_input(nbio_sock_t socket)
 
   DEBUG(2, Sdprintf("[%d]: nbio_close_input(%d, flags=0x%x)\n",
 		    PL_thread_self(), socket, s->flags));
-  s->input = NULL;
   s->flags &= ~SOCK_INSTREAM;
 #ifdef __WINDOWS__
   if ( false(s, SOCK_LISTEN) )
@@ -2185,11 +2204,13 @@ nbio_close_input(nbio_sock_t socket)
     { DEBUG(1, Sdprintf("shutdown(%d=%d, SD_RECEIVE) failed: %s\n",
 			socket, s->socket,
 			WinSockError(WSAGetLastError())));
+      Sseterr(s->input, SIO_FERR, WinSockError(WSAGetLastError()));
+      rc = -1;
     }
-    rc = -1;
   }
 #endif
 
+  s->input = NULL;
   if ( !(s->flags & (SOCK_INSTREAM|SOCK_OUTSTREAM)) )
     return freeSocket(s);
 
@@ -2213,25 +2234,23 @@ nbio_close_output(nbio_sock_t socket)
     SOCKET sock;
 #endif
 
-    s->output = NULL;
     s->flags &= ~SOCK_OUTSTREAM;
 #if __WINDOWS__
     if ( (sock=s->socket) < 0 )
     { s->error = WSAECONNRESET;
       rc = -1;
     } else if ( shutdown(sock, SD_SEND) == SOCKET_ERROR )
-    {
+    { const char *msg;
+#ifdef __WINDOWS__
+      msg = WinSockError(WSAGetLastError());
+#else
+      msg = strerror(errno);
+#endif
+      Sseterr(s->output, SIO_FERR, msg);
 #ifdef O_DEBUG
       if ( debugging )
-      { const char *msg;
-#ifdef __WINDOWS__
-	msg = WinSockError(WSAGetLastError());
-#else
-        msg = strerror(errno);
-#endif
-        Sdprintf("shutdown(%d=%d, SD_SEND) failed: %s\n",
+	Sdprintf("shutdown(%d=%d, SD_SEND) failed: %s\n",
 		 socket, s->socket, msg);
-      }
 #endif
       rc = -1;
     }
@@ -2239,6 +2258,7 @@ nbio_close_output(nbio_sock_t socket)
   }
 
   DEBUG(3, Sdprintf("%d->flags = 0x%x\n", socket, s->flags));
+  s->output = NULL;
   if ( !(s->flags & (SOCK_INSTREAM|SOCK_OUTSTREAM)) )
     return freeSocket(s);
 
@@ -2321,7 +2341,9 @@ ssize_t
 nbio_sendto(nbio_sock_t socket, void *buf, size_t bufSize, int flags,
 	    const struct sockaddr *to, socklen_t tolen)
 { plsocket *s;
+#ifdef __WINDOWS__
   ssize_t n;
+#endif
 
   if ( !(s = nbio_to_plsocket(socket)) )
     return -1;
