@@ -58,6 +58,7 @@
 
 static atom_t ATOM_reuseaddr;		/* "reuseaddr" */
 static atom_t ATOM_broadcast;		/* "broadcast" */
+static atom_t ATOM_nodelay;		/* "nodelay" */
 static atom_t ATOM_dispatch;		/* "dispatch" */
 static atom_t ATOM_nonblock;		/* "nonblock" */
 static atom_t ATOM_infinite;		/* "infinite" */
@@ -159,6 +160,25 @@ pl_setopt(term_t Socket, term_t opt)
     { if ( nbio_setopt(socket, TCP_REUSEADDR, TRUE) == 0 )
 	return TRUE;
 
+      return FALSE;
+    } else if ( a == ATOM_nodelay && arity <= 1 )
+    { int enable, rc;
+
+      if ( arity == 0 )
+      { enable = TRUE;
+      } else /*if ( arity == 1 )*/
+      { term_t a = PL_new_term_ref();
+
+	PL_get_arg(1, opt, a);
+	if ( !PL_get_bool(a, &enable) )
+	  return pl_error(NULL, 0, NULL, ERR_DOMAIN, a, "boolean");
+      }
+
+      if ( (rc=nbio_setopt(socket, TCP_NO_DELAY, enable) == 0) )
+	return TRUE;
+      if ( rc == -2 )
+	goto not_implemented;
+
       return FALSE;
     } else if ( a == ATOM_broadcast && arity == 0 )
     { if ( nbio_setopt(socket, UDP_BROADCAST, TRUE) == 0 )
@@ -181,6 +201,7 @@ pl_setopt(term_t Socket, term_t opt)
     }
   }
        
+not_implemented:
   return pl_error(NULL, 0, NULL, ERR_DOMAIN, opt, "socket_option");
 }
 
@@ -543,18 +564,24 @@ pl_accept(term_t Master, term_t Slave, term_t Peer)
 
 static foreign_t
 pl_gethostname(term_t name)
-{ char buf[256];
+{ static atom_t hname;
 
-  if ( gethostname(buf, sizeof(buf)) == 0 )
-  { struct hostent *he;
+  if ( !hname )
+  { char buf[256];
 
-    if ( (he = gethostbyname(buf)) )
-      return PL_unify_atom_chars(name, he->h_name);
-    else
-      return PL_unify_atom_chars(name, buf);
+    if ( gethostname(buf, sizeof(buf)) == 0 )
+    { struct hostent *he;
+      
+      if ( (he = gethostbyname(buf)) )
+	hname = PL_new_atom(he->h_name);
+      else
+	hname = PL_new_atom(buf);
+    } else
+    { return nbio_error(h_errno, TCP_HERRNO);
+    }
   }
 
-  return nbio_error(h_errno, TCP_HERRNO);
+  return PL_unify_atom(name, hname);
 }
 
 
@@ -691,7 +718,7 @@ tcp_select(term_t Streams, term_t Available, term_t timeout)
 
   switch(ret)
   { case -1:
-      return pl_error("tcp_select", 3, ERR_ERRNO, errno);
+      return pl_error("tcp_select", 3, ERR_ERRNO, errno, "select", "streams", Streams);
 
     case 0: /* Timeout */
       break;
@@ -730,6 +757,7 @@ install_socket()
 
   ATOM_reuseaddr  = PL_new_atom("reuseaddr");
   ATOM_broadcast  = PL_new_atom("broadcast");
+  ATOM_nodelay    = PL_new_atom("nodelay");
   ATOM_dispatch   = PL_new_atom("dispatch");
   ATOM_nonblock   = PL_new_atom("nonblock");
   ATOM_infinite   = PL_new_atom("infinite");
