@@ -3,7 +3,7 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@uva.nl
     WWW:           http://www.swi-prolog.org
     Copyright (C): 1985-2008, University of Amsterdam
 
@@ -53,6 +53,7 @@
 :- use_module(library(url)).
 :- use_module(library(memfile)).
 :- use_module(library(settings)).
+:- use_module(library(error)).
 :- use_module(dcg_basics).
 :- use_module(html_write).
 :- use_module(mimetype).
@@ -82,7 +83,12 @@ http_read_request(In, Request) :-
 	    Request = end_of_file
 	;   debug(http(header), 'First line: ~s~n', [Codes]),
 	    Request =  [input(In)|Request1],
-	    phrase(request(In, Request1), Codes)
+	    phrase(request(In, Request1), Codes),
+	    (	Request1 = [unknown(Text)|_]
+	    ->	string_to_list(S, Text),
+		domain_error(http_request, S)
+	    ;	true
+	    )
 	).
 
 
@@ -179,7 +185,7 @@ http_status_reply(moved(To), Out, HrdExtra) :- !,
 		      p(['The document has moved ',
 			 a(href(To), ' Here')
 			]),
-		      address(httpd)
+		      \address
 		    ]),
 	       HTML),
 	phrase(reply_header(moved(To, HTML), HrdExtra), Header),
@@ -192,7 +198,7 @@ http_status_reply(moved_temporary(To), Out, HrdExtra) :- !,
 		      p(['The document is currently ',
 			 a(href(To), ' Here')
 			]),
-		      address(httpd)
+		      \address
 		    ]),
 	       HTML),
 	phrase(reply_header(moved_temporary(To, HTML), HrdExtra), Header),
@@ -205,7 +211,7 @@ http_status_reply(see_other(To),Out,HdrExtra) :- !,
                       p(['See other document ',
                          a(href(To), ' Here')
                         ]),
-                      address(httpd)
+                      \address
                     ]),
                HTML),
         phrase(reply_header(see_other(To, HTML), HdrExtra), Header),
@@ -218,7 +224,7 @@ http_status_reply(not_found(URL), Out, HrdExtra) :- !,
 		      p(['The requested URL ', tt(URL),
 			 ' was not found on this server'
 			]),
-		      address(httpd)
+		      \address
 		    ]),
 	       HTML),
 	phrase(reply_header(status(not_found, HTML), HrdExtra), Header),
@@ -231,7 +237,7 @@ http_status_reply(forbidden(URL), Out, HrdExtra) :- !,
 		      p(['You don\'t have permission to access ', URL,
 			 ' on this server'
 			]),
-		      address(httpd)
+		      \address
 		    ]),
 	       HTML),
 	phrase(reply_header(status(forbidden, HTML), HrdExtra), Header),
@@ -248,59 +254,44 @@ http_status_reply(authorise(Method, Realm), Out, HrdExtra) :- !,
 			 'browser doesn\'t understand how to supply ',
 			 'the credentials required.'
 			]),
-		      address(httpd)
+		      \address
 		    ]),
 	       HTML),
 	phrase(reply_header(authorise(Method, Realm, HTML), HrdExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
 http_status_reply(not_modified, Out, HrdExtra) :- !,
-	phrase(page([ title('304 Not Modified')
-		    ],
-		    [ h1('Not Modified'),
-		      p(['The resource has not changed']),
-		      address(httpd)
-		    ]),
-	       HTML),
-	phrase(reply_header(status(not_modified, HTML), HrdExtra), Header),
-	format(Out, '~s', [Header]),
-	print_html(Out, HTML).
+	phrase(reply_header(status(not_modified), HrdExtra), Header),
+	format(Out, '~s', [Header]).
 http_status_reply(server_error(ErrorTerm), Out, HrdExtra) :-
 	'$messages':translate_message(ErrorTerm, Lines, []),
 	phrase(page([ title('500 Internal server error')
 		    ],
 		    [ h1('Internal server error'),
 		      p(\html_message_lines(Lines)),
-		      address(httpd)
+		      \address
 		    ]),
 	       HTML),
 	phrase(reply_header(status(server_error, HTML), HrdExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
-http_status_reply(resource_error(ErrorTerm), Out, HrdExtra) :- !,
+http_status_reply(unavailable(WhyHTML), Out, HdrExtra) :- !,
+	phrase(page([ title('503 Service Unavailable')
+		    ],
+		    [ h1('Service Unavailable'),
+		      WhyHTML,
+		      \address
+		    ]),
+	       HTML),
+	phrase(reply_header(status(service_unavailable, HTML), HdrExtra), Header),
+	format(Out, '~s', [Header]),
+	print_html(Out, HTML).
+http_status_reply(resource_error(ErrorTerm), Out, HdrExtra) :- !,
 	'$messages':translate_message(ErrorTerm, Lines, []),
-	phrase(page([ title('503 Service Unavailable')
-		    ],
-		    [ h1('Service Unavailable'),
-		      p(['The server is temporarily out of resources, please try again later']),
-		      p(\html_message_lines(Lines)),
-		      address(httpd)
-		    ]),
-	       HTML),
-	phrase(reply_header(status(service_unavailable, HTML), HrdExtra), Header),
-	format(Out, '~s', [Header]),
-	print_html(Out, HTML).
-http_status_reply(busy, Out, HrdExtra) :- !,
-	phrase(page([ title('503 Service Unavailable')
-		    ],
-		    [ h1('Service Unavailable'),
-		      p(['The server is temporarily out of resources, please try again later']),
-		      address(httpd)
-		    ]),
-	       HTML),
-	phrase(reply_header(status(service_unavailable, HTML), HrdExtra), Header),
-	format(Out, '~s', [Header]),
-	print_html(Out, HTML).
+	http_status_reply(unavailable(p(\html_message_lines(Lines))), Out, HdrExtra).
+http_status_reply(busy, Out, HdrExtra) :- !,
+	HTML = p(['The server is temporarily out of resources, please try again later']),
+	http_status_reply(unavailable(HTML), Out, HdrExtra).
 
 
 html_message_lines([]) -->
@@ -426,7 +417,7 @@ http_update_transfer(Request, CgiHeader, Transfer, Header) :-
 	http_update_transfer(When, Request, CgiHeader, Transfer, Header).
 
 http_update_transfer(never, _, CgiHeader, none, Header) :- !,
-	delete(transfer_encoding(_), CgiHeader, Header).
+	delete(CgiHeader, transfer_encoding(_), Header).
 http_update_transfer(_, Request, CgiHeader, Transfer, Header) :-
 	select(transfer_encoding(CgiTransfer), CgiHeader, Rest), !,
 	transfer(Request, ReqConnection),
@@ -721,6 +712,10 @@ reply_header(see_other(To,Tokens),HdrExtra) -->
 	content_length(html(Tokens)),
 	content_type(text/html, utf8),
 	"\r\n".
+reply_header(status(Status), HdrExtra) --> % Empty messages: 1xx, 204 and 304
+	vstatus(Status),
+	header_fields(HdrExtra),
+	"\r\n".
 reply_header(status(Status, Tokens), HdrExtra) -->
 	vstatus(Status),
 	date(now),
@@ -894,7 +889,7 @@ field_to_prolog(set_cookie, ValueChars, SetCookie) :- !,
 	debug(cookie, 'SetCookie: ~s', [ValueChars]),
 	phrase(set_cookie(SetCookie), ValueChars).
 field_to_prolog(host, ValueChars, Host) :- !,
-	(   append(HostChars, [0':|PortChars], ValueChars),
+	(   append(HostChars, [0':|PortChars], ValueChars), % 0'
 	    catch(number_codes(Port, PortChars), _, fail)
 	->  atom_codes(HostName, HostChars),
 	    Host = HostName:Port
@@ -937,7 +932,17 @@ header_fields([H|T]) -->
 
 %%	field_name(?PrologName)
 %
-%	Convert between prolog_name and HttpName
+%	Convert between prolog_name and HttpName.  Field names are,
+%	aoording to RFC 2616, considered tokens and covered by the
+%	following definition:
+%	
+%	==
+%	token          = 1*<any CHAR except CTLs or separators>
+%       separators     = "(" | ")" | "<" | ">" | "@"
+%                      | "," | ";" | ":" | "\" | <">
+%                      | "/" | "[" | "]" | "?" | "="
+%                      | "{" | "}" | SP | HT
+%	==
 
 field_name(Name) -->
 	{ var(Name) }, !,
@@ -956,19 +961,26 @@ rd_field_chars([C0|T]) -->
 rd_field_chars([]) -->
 	[].
 
+%%	separators(-CharCodes) is det.
+%
+%	CharCodes is a list of separators according to RFC2616
+
+separators("()<>@,;:\\\"/[]?={} \t").	% \"
+
 term_expansion(rd_field_char(_,_), Clauses) :-
 	Clauses = [ rd_field_char(0'-, 0'_)
 		  | Cls
 		  ],
+	separators(Seps),
 	findall(rd_field_char(In, Out),
-		(   between(1, 127, In),
-		    code_type(In, csymf),
+		(   between(32, 127, In),
+		    \+ memberchk(In, Seps),
+		    In \== 0'-,		% 0'
 		    code_type(Out, to_lower(In))),
 		Cls).
 
 rd_field_char(_, _).
 
-
 wr_field_chars([C|T]) -->
 	[C2], !,
 	{ to_lower(C2, C) },
@@ -976,7 +988,7 @@ wr_field_chars([C|T]) -->
 wr_field_chars([]) -->
 	[].
 
-wr_field_chars2([0'_|T]) --> !,
+wr_field_chars2([0'_|T]) --> !,		% 0'
 	"-",
 	wr_field_chars(T).
 wr_field_chars2([C|T]) --> !,
@@ -1098,7 +1110,7 @@ cookie_value(Value) -->
 	}.
 
 chars_to_semicolon_or_blank([]) -->
-	peek(0';), !.
+	peek(0';), !.			% 0'
 chars_to_semicolon_or_blank([]) -->
 	blank, !.
 chars_to_semicolon_or_blank([H|T]) -->
@@ -1125,19 +1137,30 @@ cookie_options([]) -->
 	blanks.
 
 
-cookie_option(secure=true) -->
-	"secure", !.
+%%	cookie_option(-Option)// is semidet.
+%
+%	True if input represents a valid  Cookie option. Officially, all
+%	cookie  options  use  the  syntax   <name>=<value>,  except  for
+%	=secure=.  M$  decided  to  extend  this  to  include  at  least
+%	=httponly= (only the Gods know what it means).
+%	
+%	@param	Option	Term of the form Name=Value
+%	@bug	Incorrectly accepts options without = for M$ compatibility.
+
 cookie_option(Name=Value) -->
 	rd_field_chars(NameChars), whites,
-	"=", blanks,
-	chars_to_semicolon(ValueChars),
-	{ atom_codes(Name, NameChars),
-	  atom_codes(Value, ValueChars)
-	}.
+	{ atom_codes(Name, NameChars) },
+	(   "="
+	->  blanks,
+	    chars_to_semicolon(ValueChars),
+	    { atom_codes(Value, ValueChars)
+	    }
+	;   { Value = true }
+	).
 
 chars_to_semicolon([]) -->
 	blanks,
-	peek(0';), !.
+	peek(0';), !.			% 0'
 chars_to_semicolon([H|T]) -->
 	[H], !,
 	chars_to_semicolon(T).
@@ -1217,6 +1240,27 @@ header([]) -->
 	blanks,
 	[].
 
+%%	address//
+%
+%	Emit the HTML for the server address on behalve of error and
+%	status messages (non-200 replies).  Default is
+%
+%	    ==
+%	    SWI-Prolog httpd at <hostname>
+%	    ==
+%
+%	The address can be modified by   providing  a definition for the
+%	multifile predicate http:http_address//0.
+
+:- multifile
+	http:http_address//0.
+
+address -->
+	http:http_address, !.
+address -->
+	{ gethostname(Host) },
+	html(address(['SWI-Prolog httpd at ', Host])).
+
 mkfield(host, Host:Port, [host(Host),port(Port)|Tail], Tail) :- !.
 mkfield(Name, Value, [Att|Tail], Tail) :-
 	Att =.. [Name, Value].
