@@ -3,9 +3,9 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2007, University of Amsterdam
+    Copyright (C): 2008, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -41,8 +41,9 @@
 :- use_module(library(option)).
 :- use_module(library(error)).
 :- use_module(library(base64)).
+:- use_module(library(debug)).
 
-user_agent('SWI-Prolog (http://www.swi-prolog.org)').
+user_agent('SWI-Prolog <http://www.swi-prolog.org>').
 
 /** <module> Simple HTTP client
 
@@ -89,12 +90,12 @@ client support is provided by http_client.pl
 
 http_open(URL, Stream, Options) :-
 	atom(URL), !,
-	parse_url(URL, Parts),
+	parse_url_ex(URL, Parts),
 	add_authorization(URL, Options, Options1),
 	http_open(Parts, Stream, Options1).
 http_open(Parts, Stream, Options0) :-
 	memberchk(proxy(Host, ProxyPort), Options0), !,
-	parse_url(Location, Parts),
+	parse_url_ex(Location, Parts),
 	Options = [visited(Parts)|Options0],
 	open_socket(Host:ProxyPort, In, Out, Options),
 	option(port(Port), Parts, 80),
@@ -215,19 +216,19 @@ do_open(200, _, Lines, Options, Parts, In0, In) :- !,
 	return_fields(Options, Lines),
 	transfer_encoding_filter(Lines, In0, In),
 					% properly re-initialise the stream
-	parse_url(Id, Parts),
+	parse_url_ex(Id, Parts),
 	set_stream(In, file_name(Id)),
 	set_stream(In, record_position(true)).
 					% Handle redirections
 do_open(Code, _, Lines, Options, Parts, In, Stream) :-
 	redirect_code(Code),
 	location(Lines, Location), !,
-	parse_url(Location, Parts, Redirected),
+	parse_url_ex(Location, Parts, Redirected),
 	close(In),
 	http_open(Redirected, Stream, [visited(Redirected)|Options]).
 					% report anything else as error
 do_open(Code, Comment, _,  _, Parts, _, _) :-
-	parse_url(Id, Parts),
+	parse_url_ex(Id, Parts),
 	(   map_error_code(Code, Error)
 	->  Formal =.. [Error, url, Id]
 	;   Formal = existence_error(url, Id)
@@ -252,10 +253,27 @@ redirect_code(301).			% moved permanently
 redirect_code(302).			% moved temporary
 redirect_code(303).			% see also
 
-open_socket(Host:Port, In, Out, Options) :-
+%%	open_socket(+Address, -In, -Out, +Options) is det.
+%
+%	Create and connect a client socket to Address.  Options
+%	
+%	    * timeout(+Timeout)
+%	    Sets timeout on the stream, *after* connecting the
+%	    socket.
+%	    
+%	@tbd	Make timeout also work on tcp_connect/2.
+%	@tbd	This is the same as do_connect/4 in http_client.pl
+
+open_socket(Address, In, Out, Options) :-
+	debug(http(open), 'http_open: Connecting to ~p ...', [Address]),
 	tcp_socket(Socket),
-	tcp_connect(Socket, Host:Port),
+	catch(tcp_connect(Socket, Address),
+	      E,
+	      (	  tcp_close_socket(Socket),
+		  throw(E)
+	      )),
 	tcp_open_socket(Socket, In, Out),
+	debug(http(open), '\tok ~p --> ~p', [In, Out]),
 	set_stream(In, record_position(false)),
 	(   memberchk(Options, timeout(Timeout))
 	->  set_stream(In, timeout(Timeout))
@@ -290,7 +308,7 @@ return_final_url(Options) :-
 	memberchk(final_url(URL), Options),
 	var(URL), !,
 	memberchk(visited(Parts), Options),
-	parse_url(URL, Parts).
+	parse_url_ex(URL, Parts).
 return_final_url(_).
 
 	
@@ -315,6 +333,7 @@ transfer_encoding_filter(_, In, In).
 transfer_encoding(Lines, Encoding) :-
 	member(Line, Lines),
 	phrase(transfer_encoding(Encoding0), Line), !,
+	debug(http(transfer_encoding), 'Transfer-encoding: ~w', [Encoding0]),
 	Encoding = Encoding0.
 
 transfer_encoding(Encoding) -->
@@ -485,9 +504,19 @@ add_authorization(For, Options0, Options) :-
 	stored_authorization(_, _) ->	% quick test to avoid work
 	(   atom(For)
 	->  URL = For
-	;   parse_url(URL, For)
+	;   parse_url_ex(URL, For)
 	),
 	authorization(URL, Auth), !,
 	Options = [authorization(Auth)|Options0].
 add_authorization(_, Options, Options).
 	
+
+parse_url_ex(URL, Parts) :-
+	parse_url(URL, Parts), !.
+parse_url_ex(URL, _) :-
+	domain_error(url, URL).		% Syntax error?
+
+parse_url_ex(URL, RelativeTo, Parts) :-
+	parse_url(URL, RelativeTo, Parts), !.
+parse_url_ex(URL, _, _) :-
+	domain_error(url, URL).		% Syntax error?
