@@ -42,6 +42,7 @@
 :- use_module(library(lists)).
 :- use_module(library(debug)).
 :- use_module(library(memfile)).
+:- use_module(library(pairs)).
 
 
 		 /*******************************
@@ -139,7 +140,9 @@ list_item(Lines, _, Indent, [SubList|LIT], LIT, Rest) :-	% sub-list
 %	Extract the remainder (after the first line) of a list item.
 
 rest_list_item([], _, _, [], []).
-rest_list_item([_-[]|L], _, _, [], L) :- !.	% empty line
+rest_list_item([_-[]|RestLines], _, N, Pars, Rest) :-	% empty line
+	take_pars_at_indent(RestLines, N, Pars, Rest).
+rest_list_item([_-[]|Ln], _, _, [], Ln) :- !.
 rest_list_item(L, _, N, [], L) :-		% less indented
 	L = [I-_|_], I < N, !.
 rest_list_item(L, _, _, [], L) :-		% Start with mark
@@ -149,6 +152,14 @@ rest_list_item([_-L1|L0], Type, N, ['\n'|LI], L) :-
 	append(L1, LIT, LI),
 	rest_list_item(L0, Type, N, LIT, L).
 
+%%	take_pars_at_indent(+Lines, +Indent, -Pars, -RestLines) is det.
+%
+%	Process paragraphs in bullet-lists.
+
+take_pars_at_indent(Lines, N, [Par|RestPars], RestLines) :-
+	take_par(Lines, Par, RL), Par = p(_), !,
+	take_pars_at_indent(RL, N, RestPars, RestLines).
+take_pars_at_indent(Lines, _, [], Lines).
 
 %%	rest_list(+Lines, +Type, +Indent,
 %%		  -Items, -ItemTail, -RestLines) is det.
@@ -327,7 +338,8 @@ strip_leading_ws(T, T).
 tags(Lines, Tags) :-
 	collect_tags(Lines, Tags0),
 	keysort(Tags0, Tags1),
-	combine_tags(Tags1, Tags).
+	pairs_values(Tags1, Tags2),
+	combine_tags(Tags2, Tags).
 
 %%	collect_tags(+IndentedLines, -Tags) is det
 %
@@ -383,44 +395,49 @@ renamed_tag(exception, throws).
 %	versions may issue a warning if the order is inconsistent.
 
 tag_order(param,       1).
-tag_order(throws,      2).
 tag_order(error,       2).		% same as throw
-tag_order(author,      3).
-tag_order(version,     4).
-tag_order(see,	       5).
-tag_order(deprecated,  6).
-tag_order(compat,      7).		% PlDoc extension
-tag_order(copyright,   8).
-tag_order(license,     9).
-tag_order(bug,	      10).
-tag_order(tbd,	      11).
+tag_order(throws,      3).
+tag_order(author,      4).
+tag_order(version,     5).
+tag_order(see,	       6).
+tag_order(deprecated,  7).
+tag_order(compat,      8).		% PlDoc extension
+tag_order(copyright,   9).
+tag_order(license,    10).
+tag_order(bug,	      11).
+tag_order(tbd,	      12).
 
 
-%%	combine_tags(+TaggedTags:list, -Tags:list) is det.
+%%	combine_tags(+Tags:list(tag(Key, Value)), -Tags:list) is det.
 %
 %	Creates the final tag-list.  Tags is a list of
 %	
 %		* \params(list(param(Name, Descr)))
-%		* \tag(Name, Descr)
+%		* \tag(Name, list(Descr))
 %	
 %	Descr is a list of tokens.
 
 combine_tags([], []).
-combine_tags([_-tag(param, V1)|T0], [\params([P1|PL])|Tags]) :- !,
+combine_tags([tag(param, V1)|T0], [\params([P1|PL])|Tags]) :- !,
 	param_tag(V1, P1),
 	param_tags(T0, PL, T1),
 	combine_tags(T1, Tags).
-combine_tags([_-tag(Tag,Value)|T0], [\tag(Tag, Value)|T]) :-
-	combine_tags(T0, T).
+combine_tags([tag(Tag,V0)|T0], [\tag(Tag, [V0|Vs])|T]) :-
+	same_tag(Tag, T0, T1, Vs),
+	combine_tags(T1, T).
 
 param_tag([PN|Descr0], param(PN, Descr)) :-
 	strip_leading_ws(Descr0, Descr).
 
-param_tags([_-tag(param, V1)|T0], [P1|PL], T) :- !,
+param_tags([tag(param, V1)|T0], [P1|PL], T) :- !,
 	param_tag(V1, P1),
 	param_tags(T0, PL, T).
 param_tags(T, [], T).
 
+same_tag(Tag, [tag(Tag, V)|T0], T, [V|Vs]) :- !,
+	same_tag(Tag, T0, T, Vs).
+same_tag(_, L, L, []).
+
 
 		 /*******************************
 		 *	       FACES		*
@@ -636,18 +653,22 @@ url_protocol(String) :-	sub_atom(String, 0, _, 0, mailto).
 
 
 peek_end_url -->
-	peek_nl, !.
-peek_end_url -->
-	peek(' '), !.
+	peek(End),
+	{ space_atom(End) }, !.
 peek_end_url -->
 	eos, !.
 
+space_atom(' ').
+space_atom('\r').
+space_atom('\n').
+
 %%	autolink_extension(?Ext, ?Type) is nondet.
 %
 %	True if Ext is a filename extensions that create automatic links
 %	in the documentation.
 
-autolink_extension(pl, prolog).
+autolink_extension(Ext, prolog) :-
+	prolog_file_type(Ext,prolog).
 autolink_extension(txt, wiki).
 autolink_extension(gif, image).
 autolink_extension(png, image).
@@ -1022,14 +1043,6 @@ non_ws(T, T) -->
 	[].
 
 
-%%	peek_nl//
-%
-%	True if we are at the end of a line
-
-peek_nl -->
-	peek(H),
-	{ H == 0'\n ; H == 0'\r }, !.
-
 %%	nl//
 %
 %	Get end-of-line
