@@ -50,6 +50,7 @@
 	    rdf_set_predicate/2,	% +Predicate, +Property
 	    rdf_predicate_property/2,	% +Predicate, ?Property
 	    rdf_current_predicate/1,	% -Predicate
+	    rdf_current_literal/1,	% -Literal
 	    rdf_transaction/1,		% :Goal
 	    rdf_transaction/2,		% :Goal, +Id
 	    rdf_active_transaction/1,	% ?Id
@@ -107,6 +108,7 @@
 	    rdf_destroy_literal_map/1,	% +Handle
 	    rdf_reset_literal_map/1,	% +Handle
 	    rdf_insert_literal_map/3,	% +Handle, +Key, +Literal
+	    rdf_insert_literal_map/4,	% +Handle, +Key, +Literal, -NewKeys
 	    rdf_delete_literal_map/3,	% +Handle, +Key, +Literal
 	    rdf_delete_literal_map/2,	% +Handle, +Key
 	    rdf_find_literal_map/3,	% +Handle, +KeyList, -Literals
@@ -126,6 +128,7 @@
 :- use_module(library(nb_set)).
 :- use_module(library(error)).
 :- use_module(library(url)).
+:- use_module(library(debug)).
 :- use_module(rdf_cache).
 
 :- initialization
@@ -233,11 +236,14 @@ rdf_global_id(NS:Local, Global) :-
 rdf_global_id(Global, Global).
 
 
-%%	rdf_global_object(?Object, ?GlobalObject) is det.
+%%	rdf_global_object(+Object, -GlobalObject) is semidet.
+%%	rdf_global_object(-Object, +GlobalObject) is semidet.
 %	
 %	Same as rdf_global_id/2,  but  intended   for  dealing  with the
 %	object part of a  triple,  in   particular  the  type  for typed
 %	literals.
+%	
+%	@error	existence_error(rdf_namespace, NS)
 
 rdf_global_object(NS:Local, Global) :-
 	global(NS, Local, Global), !.
@@ -252,13 +258,12 @@ global(NS, Local, Global) :-
 	    atom_concat(Full, Local, Global)
 	;   atom(NS), atom(Local)
 	->  (   ns(NS, Full)
-	    *-> atom_concat(Full, Local, Global)
-	    ;   atom_concat(NS, Local, Global)
+	    *->	atom_concat(Full, Local, Global)
+	    ;	existence_error(rdf_namespace, NS)
 	    )
 	).
 
 
-
 %%	rdf_global_term(+TermIn, -GlobalTerm) is det.
 %	
 %	Does rdf_global_id/2 on all terms NS:Local by recursively analysing
@@ -267,8 +272,8 @@ global(NS, Local, Global) :-
 rdf_global_term(Var, Var) :-
 	var(Var), !.
 rdf_global_term(NS:Local, Global) :-
-	rdf_global_id(NS:Local, Global0), !,
-	Global = Global0.
+	atom(NS), atom(Local), ns(NS, Full), !,
+	atom_concat(Full, Local, Global).
 rdf_global_term([H0|T0], [H|T]) :- !,
 	rdf_global_term(H0, H),
 	rdf_global_term(T0, T).
@@ -705,6 +710,19 @@ rdf_load_db_no_admin(File, Id, Graphs) :-
 	call_cleanup(rdf_load_db_(In, Id, Graphs), close(In)).
 
 
+%%	check_loaded_cache(+DB, +Graphs, +Modified) is det.
+%
+%	Verify the loaded cache file and optionally fix the modification
+%	time (new versions save this along with the snapshot).
+%	
+%	@tbd	What to do if there is a cache mismatch? Delete the loaded
+%		graphs and fail?
+
+check_loaded_cache(DB, DB, _Modified) :- !.
+check_loaded_cache(DB, Graphs, _) :-
+	print_message(warning, rdf(inconsistent_cache(DB, Graphs))).
+
+
 %%	rdf_load_db(+File) is det.
 %
 %	Load triples from a file created using rdf_save_db/2 and update
@@ -719,8 +737,9 @@ rdf_load_db(File) :-
 	    ),
 	    rdf_md5(DB, MD5),
 	    rdf_statistics_(triples(DB, Triples)),
+	    rdf_graph_source_(DB, SourceURL, Modified),
 	    retractall(rdf_source(DB, _, _, _, _)),
-	    assert(rdf_source(DB, -, 0, Triples, MD5)),
+	    assert(rdf_source(DB, SourceURL, Modified, Triples, MD5)),
 	    fail
 	;   true
 	).
@@ -735,6 +754,7 @@ rdf_load_db(File) :-
 	rdf_load_stream/3,
 	rdf_input_info_hook/3,
 	rdf_file_type/2,
+	rdf_storage_encoding/2,
 	url_protocol/1,
 	exists_url/1.
 
@@ -814,11 +834,13 @@ rdf_load(Spec, Options0) :-
 	->  do_unload(DB),		% unload old
 	    (   Cache == true,
 		read_cache(SourceURL, Modified, CacheFile),
-	        catch(rdf_load_db_no_admin(CacheFile, cache(DB), _Graphs), _, fail)
-	    ->	Action = load
+	        catch(rdf_load_db_no_admin(CacheFile, cache(DB), Graphs), _, fail),
+		check_loaded_cache(DB, Graphs, Modified)
+	    ->  Action = load
 	    ;   rdf_input_open(Input, Stream, Format),
 		must_be(ground, Format),
 		rdf_format(Format, RDFFormat),
+		rdf_set_graph_source(DB, SourceURL, Modified),
 		call_cleanup(rdf_load_stream(RDFFormat, Stream,
 					     [ base_uri(BaseURI),
 					       blank_nodes(ShareMode),
@@ -827,7 +849,6 @@ rdf_load(Spec, Options0) :-
 					     ]),
 			     close_input(Input, Stream)), !,
 		register_file_ns(NSList),
-		rdf_set_graph_source(DB, SourceURL),
 		(   Cache == true,
 		    rdf_cache_file(SourceURL, write, CacheFile)
 		->  catch(save_cache(DB, CacheFile), E,
@@ -885,9 +906,6 @@ close_input(_, Stream) :-
 %	
 %	BaseURI is unified with a default base   URI. Note that this may
 %	be overruled from the options of rdf_load/2.
-%	
-%	@tbd	More generic mechanism to deal with encoding envelopes
-%		such as gzip compression.
 
 rdf_input(stream(Stream), stream(Stream), BaseURI) :- !,
 	(   stream_property(Stream, file_name(File))
@@ -903,7 +921,7 @@ rdf_input(FileURL, file(File), BaseURI) :-
 	file_input(File0, File, BaseURI).
 rdf_input(URL0, url(Protocol, URL), URL0) :-
 	is_url(URL0, Protocol), !,
-	(   gzip_extend(URL0, URL),
+	(   storage_extend(URL0, URL),
 	    exists_url(url(Protocol, URL))
 	->  true
 	;   URL=URL0
@@ -912,26 +930,32 @@ rdf_input(Spec, file(Path), BaseURI) :-
 	file_input(Spec, Path, BaseURI).
 
 file_input(Spec, Path, BaseURI) :-
-	findall(Ext, (rdf_file_type(Ext, _);Ext=gz;Ext=''), Exts),
+	findall(Ext, valid_extension(Ext), Exts),
 	absolute_file_name(Spec, Path,
 			   [ access(read),
-			     extensions(Exts),
-			     file_errors(fail)
+			     extensions(Exts)
 			   ]),
 	file_name_to_url(Path, BaseURI0),
 	clean_base_uri(BaseURI0, BaseURI).
 
-gzip_extend(Path, Path).
-gzip_extend(Path0, Path) :-
-	file_name_extension(Path0, gz, Path).
+valid_extension(Ext) :-
+	rdf_file_type(Ext, _).
+valid_extension(Ext) :-
+	rdf_storage_encoding(Ext, _).
+
+
+storage_extend(Path, Path).
+storage_extend(Path0, Path) :-
+	rdf_storage_encoding(Ext, _), Ext \== '',
+	file_name_extension(Path0, Ext, Path).
 
 %%	clean_base_uri(+BaseURI0, -BaseURI) is det.
 %
-%	BaseURI  is  BaseURI0  after    removing  packaging  extensions.
-%	Currently only deals with =|.gz|=.
+%	BaseURI  is  BaseURI0  after    removing  storage  extensions.
 
 clean_base_uri(BaseURI0, BaseURI) :-
-	file_name_extension(BaseURI, gz, BaseURI0), !.
+	rdf_storage_encoding(Ext, _), Ext \== '',
+	file_name_extension(BaseURI, Ext, BaseURI0), !.
 clean_base_uri(BaseURI, BaseURI).
 	
 %%	is_url(+Term, -Protocol) is semidet.
@@ -1003,6 +1027,13 @@ rdf_file_type(xhtml, xhtml).
 rdf_file_type(trp,   triples).
 
 
+%%	rdf_file_encoding(+Extension, -Format) is semidet.
+%
+%	True if Format describes the storage encoding of file.
+
+rdf_storage_encoding('', plain).
+
+
 %%	rdf_load_stream(+Format, +Stream, +Options)
 %
 %	Load RDF data from Stream.
@@ -1037,12 +1068,12 @@ read_cache(BaseURI, Modified, CacheFile) :-
 
 must_load(true, _, _) :- !.
 must_load(changed, DB, ModifiedSrc) :- !,
-	(   rdf_source(DB, _SourceURL, ModifiedLoaded, _, _)
+	(   rdf_graph_source_(DB, _SourceURL, ModifiedLoaded)
 	->  ModifiedSrc > ModifiedLoaded
 	;   true
 	).
 must_load(not_loaded, DB, _) :- !,
-	\+ rdf_source(DB, _SourceURL, _, _, _).
+	\+ rdf_source(DB).
 must_load(Cond, _, _) :-
 	throw(eror(domain_error(condition, Cond), _)).
 
@@ -1106,7 +1137,7 @@ rdf_graph(DB) :-
 
 rdf_source(DB, SourceURL) :-
 	rdf_graph(DB),
-	rdf_graph_source_(DB, SourceURL).
+	rdf_graph_source_(DB, SourceURL, _Modified).
 
 %%	rdf_source(?Source)
 %	
@@ -1213,7 +1244,8 @@ rdf_reset_db :-
 %		the full URI.  Useful to make file comparison easier.
 %	
 %	@param File	Location to save the data.  This can also be a 
-%			file-url (=|file://path|=).
+%			file-url (=|file://path|=) or a stream wrapped
+%			in a term stream(Out).
 
 :- module_transparent
 	rdf_transaction/1,
@@ -1239,13 +1271,14 @@ rdf_save(Spec, DB) :-
 	rdf_save2(File, [db(DB)]).
 
 to_file(URL, File) :-
+	atom(URL),
 	file_name_to_url(File, URL), !.
 to_file(File, File).
 
 rdf_save2(File, Options) :-
 	option(encoding(Encoding), Options, utf8),
 	valid_encoding(Encoding),
-	open(File, write, Out, [encoding(Encoding)]),
+	open_output(File, Encoding, Out, Close),
 	flag(rdf_db_saved_subjects, OSavedSubjects, 0),
 	flag(rdf_db_saved_triples, OSavedTriples, 0),
 	call_cleanup(rdf_do_save(Out, Options),
@@ -1254,8 +1287,15 @@ rdf_save2(File, Options) :-
 				  File,
 				  OSavedSubjects,
 				  OSavedTriples,
-				  Out)).
+				  Close)).
 
+open_output(stream(Out), Encoding, Out,
+	    set_stream(Out, encoding(Old))) :-
+	stream_property(Out, encoding(Old)),
+	set_stream(Out, encoding(Encoding)).
+open_output(File, Encoding, Out,
+	    close(Out)) :-
+	open(File, write, Out, [encoding(Encoding)]).
 
 valid_encoding(Enc) :-
 	(   xml_encoding_name(Enc, _)
@@ -1268,8 +1308,8 @@ cleanup_save(Reason,
 	     File,
 	     OSavedSubjects,
 	     OSavedTriples,
-	     Out) :-
-	close(Out),
+	     Close) :-
+	call(Close),
 	flag(rdf_db_saved_subjects, SavedSubjects, OSavedSubjects),
 	flag(rdf_db_saved_triples, SavedTriples, OSavedTriples),
 	retractall(named_anon(_, _)),
@@ -1342,7 +1382,8 @@ rdf_save_header(Out, Options) :-
 	(   member(Id, NSList),
 	    ns(Id, NS),
 	    rdf_quote_uri(NS, QNS),
-	    xml_quote_attribute(QNS, NSText, Enc),
+	    xml_quote_attribute(QNS, NSText0, Enc),
+	    xml_escape_parameter_entity(NSText0, NSText),
 	    format(Out, '~N    <!ENTITY ~w \'~w\'>', [Id, NSText]),
 	    fail
 	;   true
@@ -1381,6 +1422,26 @@ xml_encoding_name(iso_latin_1, 'ISO-8859-1').
 xml_encoding_name(utf8,        'UTF-8').
 
 
+%%	xml_escape_parameter_entity(+In, -Out) is det.
+%
+%	Escape % as &#37; for entity declarations.
+
+xml_escape_parameter_entity(In, Out) :-
+	sub_atom(In, _, _, _, '%'), !,
+	atom_codes(In, Codes),
+	phrase(escape_parent(Codes), OutCodes),
+	atom_codes(Out, OutCodes).
+xml_escape_parameter_entity(In, In).
+
+escape_parent([]) --> [].
+escape_parent([H|T]) -->
+	(   { H == 37 }
+	->  "&#37;"
+	;   [H]
+	),
+	escape_parent(T).
+
+
 %%	header_namespaces(Options, -List)
 %	
 %	Get namespaces we will define as entities
@@ -1460,15 +1521,18 @@ decl_used_predicate_ns(DB) :-
 decl_predicate_ns(Pred) :-
 	predicate_ns(Pred, _), !.
 decl_predicate_ns(Pred) :-
-	rdf_global_id(NS:_Local, Pred),
-	assert(predicate_ns(Pred, NS)), !.
+	rdf_global_id(NS:Local, Pred),
+	xml_name(Local), !,
+	assert(predicate_ns(Pred, NS)).
 decl_predicate_ns(Pred) :-
 	atom_codes(Pred, Codes),
 	append(NSCodes, LocalCodes, Codes),
 	xml_codes(LocalCodes), !,
 	(   NSCodes \== []
 	->  atom_codes(NS, NSCodes),
-	    (   between(1, infinite, N),
+	    (   ns(Id, NS)
+	    ->	assert(predicate_ns(Pred, Id))
+	    ;	between(1, infinite, N),
 		atom_concat(ns, N, Id),
 		\+ ns(Id, _)
 	    ->  rdf_register_ns(Id, NS),
@@ -1569,7 +1633,8 @@ rdf_save_subject(Out, Subject, BaseURI, Atts, Indent, Options) :-
 	select(RdfType=Type, Atts, Atts1),
 	rdf_id(Type, BaseURI, TypeId),
 	xml_is_name(TypeId), !,
-	format(Out, '~*|<~w', [Indent, TypeId]),
+	format(Out, '~*|<', [Indent]),
+	rdf_write_id(Out, TypeId),
 	save_about(Out, BaseURI, Subject),
 	save_attributes(Atts1, BaseURI, Out, TypeId, Indent, Options).
 rdf_save_subject(Out, Subject, BaseURI, Atts, Indent, Options) :-
@@ -1612,7 +1677,9 @@ save_attributes(Atts, BaseURI, Out, Element, Indent, Options) :-
 	->  format(Out, '/>~n', [])
 	;   format(Out, '>~n', []),
 	    save_attributes2(InBody, BaseURI, body, Out, SubIndent, Options),
-	    format(Out, '~N~*|</~w>~n', [Indent, Element])
+	    format(Out, '~N~*|</', [Indent]),
+	    rdf_write_id(Out, Element),
+	    format(Out, '>~n', [])
 	).
 
 %%	split_attributes(+Attributes, -HeadAttrs, -BodyAttr)
@@ -1681,7 +1748,9 @@ save_attribute(tag, Name=literal(Value), BaseURI, Out, Indent, _DB) :-
 	rdf_id(Name, BaseURI, NameText),
 	stream_property(Out, encoding(Encoding)),
 	xml_quote_attribute(Value, QVal, Encoding),
-	format(Out, '~N~*|~w="~w"', [AttIndent, NameText, QVal]).
+	format(Out, '~N~*|', [AttIndent]),
+	rdf_write_id(Out, NameText),
+	format(Out, '="~w"', [QVal]).
 save_attribute(body, Name=literal(Literal0), BaseURI, Out, Indent, Options) :- !,
 	rdf_id(Name, BaseURI, NameText),
 	(   memberchk(convert_typed_literal(Converter), Options),
@@ -1689,37 +1758,36 @@ save_attribute(body, Name=literal(Literal0), BaseURI, Out, Indent, Options) :- !
 	->  Literal = type(Type, Content)
 	;   Literal = Literal0
 	),
+	format(Out, '~N~*|<', [Indent]),
+	rdf_write_id(Out, NameText),
 	(   Literal = lang(Lang, Value)
 	->  (   memberchk(document_language(Lang), Options)
-	    ->  format(Out, '~N~*|<~w>', [Indent, NameText])
+	    ->  write(Out, '>')
 	    ;   rdf_id(Lang, BaseURI, LangText),
-		format(Out, '~N~*|<~w xml:lang="~w">',
-		       [Indent, NameText, LangText])
+		format(Out, ' xml:lang="~w">', [LangText])
 	    )
 	;   Literal = type(Type, Value)
 	->  (   rdf_equal(Type, rdf:'XMLLiteral')
-	    ->	format(Out, '~N~*|<~w rdf:parseType="Literal">',
-		       [Indent, NameText])
+	    ->	write(Out, ' rdf:parseType="Literal">')
 	    ;	stream_property(Out, encoding(Encoding)),
 		rdf_value(Type, BaseURI, QVal, Encoding),
-		format(Out, '~N~*|<~w rdf:datatype="~w">',
-		       [Indent, NameText, QVal])
+		format(Out, ' rdf:datatype="~w">', [QVal])
 	    )
 	;   atomic(Literal)
-	->  format(Out, '~N~*|<~w>', [Indent, NameText]),
+	->  write(Out, '>'),
 	    Value = Literal
-	;   format(Out, '~N~*|<~w rdf:parseType="Literal">',
-		   [Indent, NameText]),
+	;   write(Out, ' rdf:parseType="Literal">'),
 	    Value = Literal
 	),
 	save_attribute_value(Value, Out, Indent),
-	format(Out, '</~w>', [NameText]).
+	write(Out, '</'), rdf_write_id(Out, NameText), write(Out, '>').
 save_attribute(body, Name=Value, BaseURI, Out, Indent, Options) :-
 	rdf_is_bnode(Value), !,
 	rdf_id(Name, BaseURI, NameText),
+	format(Out, '~N~*|<', [Indent]),
+	rdf_write_id(Out, NameText),
 	(   named_anon(Value, NodeID)
-	->  format(Out, '~N~*|<~w rdf:nodeID="~w"/>',
-		   [Indent, NameText, NodeID])
+	->  format(Out, ' rdf:nodeID="~w"/>', [NodeID])
 	;   (   rdf(S1, Name, Value),
 	        rdf(S2, P2, Value),
 		(S1 \== S2 ; Name \== P2)
@@ -1730,20 +1798,23 @@ save_attribute(body, Name=Value, BaseURI, Out, Indent, Options) :-
 	    ),
 	    SubIndent is Indent + 2,
 	    (   rdf(Value, rdf:type, rdf:'List')
-	    ->  format(Out, '~N~*|<~w', [Indent, NameText]),
-		save_about(Out, BaseURI, Value),
+	    ->  save_about(Out, BaseURI, Value),
 		format(Out, ' rdf:parseType="Collection">~n', []),
 		rdf_save_list(Out, Value, BaseURI, SubIndent, Options)
-	    ;   format(Out, '~N~*|<~w>~n', [Indent, NameText]),
+	    ;   format(Out, '>~n', []),
 		rdf_save_subject(Out, Value, BaseURI, SubIndent, Options)
 	    ),
-	    format(Out, '~N~*|</~w>~n', [Indent, NameText])
+	    format(Out, '~N~*|</', [Indent]),
+	    rdf_write_id(Out, NameText),
+	    format(Out, '>~n', [])
 	).
 save_attribute(body, Name=Value, BaseURI, Out, Indent, _DB) :-
 	stream_property(Out, encoding(Encoding)),
 	rdf_value(Value, BaseURI, QVal, Encoding),
 	rdf_id(Name, BaseURI, NameText),
-	format(Out, '~N~*|<~w rdf:resource="~w"/>', [Indent, NameText, QVal]).
+	format(Out, '~N~*|<', [Indent]),
+	rdf_write_id(Out, NameText),
+	format(Out, ' rdf:resource="~w"/>', [QVal]).
 
 save_attribute_value(Value, Out, _) :-	% strings
 	atom(Value), !,
@@ -1802,6 +1873,16 @@ rdf_id(Id, _, NS:Local) :-
 rdf_id(Id, _, Id).
 
 
+%%	rdf_write_id(+Out, +NSLocal) is det.
+%
+%	Write an identifier. We cannot use native write on it as both NS
+%	and Local can be operators.
+
+rdf_write_id(Out, NS:Local) :- !,
+	format(Out, '~w:~w', [NS, Local]).
+rdf_write_id(Out, Atom) :-
+	write(Out, Atom).
+
 %%	rdf_value(+Resource, +BaseURI, -Text, +Encoding)
 %	
 %	According  to  "6.4  RDF  URI  References"  of  the  RDF  Syntax
@@ -1846,6 +1927,10 @@ prolog:message(rdf(saved(File, SavedSubjects, SavedTriples))) -->
 	].
 prolog:message(rdf(using_namespace(Id, NS))) -->
 	[ 'Using namespace id ~w for ~w'-[Id, NS] ].
+prolog:message(rdf(inconsistent_cache(DB, Graphs))) -->
+	[ 'RDF cache file for ~w contains the following graphs'-[DB], nl,
+	  '~t~8|~p'-[Graphs]
+	].
 
 how(load)   --> [ 'Loaded' ].
 how(parsed) --> [ 'Parsed' ].
