@@ -3,9 +3,9 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemak@uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2004-2007, University of Amsterdam
+    Copyright (C): 2004-2009, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@
 :- use_module(library(sgml_write)).
 :- use_module(library(assoc)).
 :- use_module(library(pairs)).
+:- use_module(library(debug)).
 
 
 /** <module> Write RDF/XML from a list of triples
@@ -47,6 +48,14 @@ format rdf(Subject, Predicate, Object).  It   is  primarily intended for
 communicating computed RDF model fragments   to  external programs using
 RDF/XML.
 
+When used from the HTTP library, use the following code:
+
+==
+reply_graph(RDF) :-
+	format('Content-type: application/rdf+xml; charset=UTF-8~n~n'),
+	rdf_write_xml(current_output, RDF).
+==
+
 @author	Jan Wielemaker
 @see	library(semweb/rdf_db) offers saving a named graph directly from
 	the RDF database.
@@ -64,7 +73,8 @@ RDF/XML.
 rdf_write_xml(Out, Triples) :-
 	sort(Triples, Unique),
 	rdf_write_header(Out, Unique),
-	rdf_write_triples(Unique, Out),
+	node_id_map(Unique, AnonIDs),
+	rdf_write_triples(Unique, AnonIDs, Out),
 	rdf_write_footer(Out).
 
 
@@ -84,7 +94,10 @@ rdf_write_header(Out, Triples) :-
 	used_namespaces(Triples, NSList),
 	(   member(Id, NSList),
 	    ns(Id, NS),
-	    format(Out, '~N    <!ENTITY ~w \'~w\'>', [Id, NS]),
+	    rdf_quote_uri(NS, QNS),
+	    xml_quote_attribute(QNS, NSText0, Encoding),
+	    xml_escape_parameter_entity(NSText0, NSText),
+	    format(Out, '~N    <!ENTITY ~w \'~w\'>', [Id, NSText]),
 	    fail
 	;   true
 	),
@@ -109,6 +122,24 @@ xml_encoding_name(ascii,       'US-ASCII').
 xml_encoding_name(iso_latin_1, 'ISO-8859-1').
 xml_encoding_name(utf8,        'UTF-8').
 
+%%	xml_escape_parameter_entity(+In, -Out) is det.
+%
+%	Escape % as &#37; for entity declarations.
+
+xml_escape_parameter_entity(In, Out) :-
+	sub_atom(In, _, _, _, '%'), !,
+	atom_codes(In, Codes),
+	phrase(escape_parent(Codes), OutCodes),
+	atom_codes(Out, OutCodes).
+xml_escape_parameter_entity(In, In).
+
+escape_parent([]) --> [].
+escape_parent([H|T]) -->
+	(   { H == 37 }
+	->  "&#37;"
+	;   [H]
+	),
+	escape_parent(T).
 
 %%	used_namespaces(+Triples:list(rdf(S,P,O)), -List:atom) is det.
 %
@@ -121,12 +152,8 @@ used_namespaces(Triples, NSList) :-
 	empty_assoc(A0),
 	put_assoc(rdf, A0, *, A1),	% needed for rdf:RDF
 	res_used_namespaces(Resources, _NoNS, A1, A),
-	assoc_to_list(A, List),
-	keys(List, NSList).
+	assoc_to_keys(A, NSList).
 
-keys([], []).
-keys([K-_|T0], [K|T]) :-
-	keys(T0, T).
 
 res_used_namespaces([], [], A, A).
 res_used_namespaces([Resource|T], NoNS, A0, A) :-
@@ -156,7 +183,7 @@ resources([rdf(S,P,O)|T]) -->
 object_resources(Atom) -->
 	{ atom(Atom) }, !,
 	[ Atom ].
-object_resources(literal(type(Type, _))) -->
+object_resources(literal(type(Type, _))) --> !,
 	[ Type ].
 object_resources(_) -->
 	[].
@@ -182,13 +209,17 @@ decl_predicate_ns(Pred) :-
 decl_predicate_ns(Pred) :-
 	rdf_global_id(NS:_Local, Pred),
 	assert(predicate_ns(Pred, NS)), !.
+decl_predicate_ns(Pred) :-
+	is_bag_li_predicate(Pred), !.
 decl_predicate_ns(Pred) :-
 	atom_codes(Pred, Codes),
 	append(NSCodes, LocalCodes, Codes),
 	xml_codes(LocalCodes), !,
 	(   NSCodes \== []
 	->  atom_codes(NS, NSCodes),
-	    (   between(1, infinite, N),
+	    (   ns(Id, NS)
+	    ->	assert(predicate_ns(Pred, Id))
+	    ;	between(1, infinite, N),
 		atom_concat(ns, N, Id),
 		\+ ns(Id, _)
 	    ->  rdf_register_ns(Id, NS),
@@ -213,22 +244,61 @@ rdf_write_footer(Out) :-
 	format(Out, '</rdf:RDF>~n', []).
 
 
+		 /*******************************
+		 *	    ANONYMOUS IDS	*
+		 *******************************/
+
+%%	node_id_map(+Triples, -IdMap) is det.
+%
+%	Create an assoc Resource -> NodeID for those anonymous resources
+%	in Triples that need  a  NodeID.   This  implies  all  anonymous
+%	resources that are used multiple times as object value.
+
+node_id_map(Triples, IdMap) :-
+	anonymous_objects(Triples, Objs),
+	msort(Objs, Sorted),
+	empty_assoc(IdMap0),
+	nodeid_map(Sorted, 0, IdMap0, IdMap).
+
+anonymous_objects([], []).
+anonymous_objects([rdf(_,_,O)|T0], Anon) :-
+	rdf_is_bnode(O), !,
+	Anon = [O|T],
+	anonymous_objects(T0, T).
+anonymous_objects([_|T0], T) :-
+	anonymous_objects(T0, T).
+
+nodeid_map([], _, Map, Map).
+nodeid_map([H,H|T0], Id, Map0, Map) :- !,
+	remove_leading(H, T0, T),
+	atom_concat(bn, Id, NodeId),
+	put_assoc(H, Map0, NodeId, Map1),
+	Id2 is Id + 1,
+	nodeid_map(T, Id2, Map1, Map).
+nodeid_map([_|T], Id, Map0, Map) :-
+	nodeid_map(T, Id, Map0, Map).
+
+remove_leading(H, [H|T0], T) :- !,
+	remove_leading(H, T0, T).
+remove_leading(_, T, T).
+
+
 		 /*******************************
 		 *	      TRIPLES		*
 		 *******************************/
 
-rdf_write_triples(Triples, Out) :-
-	rdf_write_triples(Triples, Out, [], Anon),
-	rdf_write_anon(Anon, Out, Anon).
+rdf_write_triples(Triples, NodeIDs, Out) :-
+	rdf_write_triples(Triples, NodeIDs, Out, [], Anon),
+	rdf_write_anon(Anon, NodeIDs, Out, Anon).
 
-rdf_write_triples([], _, Anon, Anon).
-rdf_write_triples([H|T0], Out, Anon0, Anon) :-
+rdf_write_triples([], _, _, Anon, Anon).
+rdf_write_triples([H|T0], NodeIDs, Out, Anon0, Anon) :-
 	arg(1, H, S),
 	subject_triples(S, [H|T0], T, OnSubject),
-	(   anonymous_subject(S)
-	->  rdf_write_triples(T, Out, [anon(S,_,OnSubject)|Anon0], Anon)
-	;   rdf_write_subject(OnSubject, S, Out, Anon0),
-	    rdf_write_triples(T, Out, Anon0, Anon)
+	(   rdf_is_bnode(S)
+	->  rdf_write_triples(T, NodeIDs, Out, [anon(S,_,OnSubject)|Anon0], Anon)
+	;   rdf_write_subject(OnSubject, S, NodeIDs, Out, Anon0),
+	    rdf_write_triples(T, NodeIDs, Out, Anon0, Anon)
 	).
 
 subject_triples(S, [H|T0], T, [H|M]) :-
@@ -237,59 +307,67 @@ subject_triples(S, [H|T0], T, [H|M]) :-
 subject_triples(_, T, T, []).
 
 
-rdf_write_anon([], _, _).
-rdf_write_anon([anon(Subject, Done, Triples)|T], Out, Anon) :-
+rdf_write_anon([], _, _, _).
+rdf_write_anon([anon(Subject, Done, Triples)|T], NodeIDs, Out, Anon) :-
 	Done \== true, !,
-	rdf_write_subject(Triples, Subject, Out, Anon),
-	rdf_write_anon(T, Out, Anon).
-rdf_write_anon([_|T], Out, Anon) :-
-	rdf_write_anon(T, Out, Anon).
+	Done = true,
+	rdf_write_subject(Triples, Subject, NodeIDs, Out, Anon),
+	rdf_write_anon(T, NodeIDs, Out, Anon).
+rdf_write_anon([_|T], NodeIDs, Out, Anon) :-
+	rdf_write_anon(T, NodeIDs, Out, Anon).
 
-rdf_write_subject(Triples, Subject, Out, Anon) :-
-	rdf_write_subject(Triples, Out, Subject, -, 0, Anon), !,
+rdf_write_subject(Triples, Subject, NodeIDs, Out, Anon) :-
+	rdf_write_subject(Triples, Out, Subject, NodeIDs, -, 0, Anon), !,
 	format(Out, '~n', []).
-rdf_write_subject(_, Subject, _, _) :-
+rdf_write_subject(_, Subject, _, _, _) :-
 	throw(error(rdf_save_failed(Subject), 'Internal error')).
 
-rdf_write_subject(Triples, Out, Subject, DefNS, Indent, Anon) :-
+rdf_write_subject(Triples, Out, Subject, NodeIDs, DefNS, Indent, Anon) :-
 	rdf_equal(rdf:type, RdfType),
 	select(rdf(_, RdfType,Type), Triples, Triples1),
 	rdf_id(Type, DefNS, TypeId),
 	xml_is_name(TypeId), !,
-	format(Out, '~*|<~w', [Indent, TypeId]),
-	save_about(Out, Subject),
-	save_attributes(Triples1, DefNS, Out, TypeId, Indent, Anon).
-rdf_write_subject(Triples, Out, Subject, _DefNS, Indent, Anon) :-
+	format(Out, '~*|<', [Indent]),
+	rdf_write_id(Out, TypeId),
+	save_about(Out, Subject, NodeIDs),
+	save_attributes(Triples1, DefNS, Out, NodeIDs, TypeId, Indent, Anon).
+rdf_write_subject(Triples, Out, Subject, NodeIDs, _DefNS, Indent, Anon) :-
 	format(Out, '~*|<rdf:Description', [Indent]),
-	save_about(Out, Subject),
-	save_attributes(Triples, rdf, Out, rdf:'Description', Indent, Anon).
+	save_about(Out, Subject, NodeIDs),
+	save_attributes(Triples, rdf, Out, NodeIDs, rdf:'Description', Indent, Anon).
 
 xml_is_name(_NS:Atom) :- !,
 	xml_name(Atom).
 xml_is_name(Atom) :-
 	xml_name(Atom).
 
-save_about(_Out, Subject) :-
-	anonymous_subject(Subject), !.
-save_about(Out, Subject) :-
+save_about(Out, Subject, NodeIDs) :-
+	rdf_is_bnode(Subject), !,
+	(   get_assoc(Subject, NodeIDs, NodeID)
+	->  format(Out,' rdf:nodeID="~w"', [NodeID])
+	;   true
+	).
+save_about(Out, Subject, _) :-
 	stream_property(Out, encoding(Encoding)),
 	rdf_value(Subject, QSubject, Encoding),
-	format(Out, ' rdf:about="~w"', [QSubject]).
+	format(Out, ' rdf:about="~w"', [QSubject]), !.
+save_about(_, _, _) :-
+	assertion(fail).
 
-%	save_attributes(+List, +DefNS, +Out, Element)
+%%	save_attributes(+List, +DefNS, +Out, +NodeIDs, Element, +Indent, +Anon)
 %
 %	Save the attributes.  Short literal attributes are saved in the
 %	tag.  Others as the content of the description element.  The
 %	begin tag has already been filled.
 
-save_attributes(Triples, DefNS, Out, Element, Indent, Anon) :-
+save_attributes(Triples, DefNS, Out, NodeIDs, Element, Indent, Anon) :-
 	split_attributes(Triples, InTag, InBody),
 	SubIndent is Indent + 2,
-	save_attributes2(InTag, DefNS, tag, Out, SubIndent, Anon),
+	save_attributes2(InTag, DefNS, tag, Out, NodeIDs, SubIndent, Anon),
 	(   InBody == []
 	->  format(Out, '/>~n', [])
 	;   format(Out, '>~n', []),
-	    save_attributes2(InBody, _, body, Out, SubIndent, Anon),
+	    save_attributes2(InBody, _, body, Out, NodeIDs, SubIndent, Anon),
 	    format(Out, '~N~*|</~w>~n', [Indent, Element])
 	).
 
@@ -340,72 +418,86 @@ simple_literal_attributes([H|TA], [H|TI], B) :-
 simple_literal_attributes([H|TA], I, [H|TB]) :-
 	simple_literal_attributes(TA, I, TB).
 
-in_tag_attribute(rdf(_,_,literal(Text))) :-
+in_tag_attribute(rdf(_,P,literal(Text))) :-
 	atom(Text),			% may not have lang qualifier
 	atom_length(Text, Len),
-	Len < 60.
+	Len < 60,
+	\+ is_bag_li_predicate(P).
 
-%	save_attributes(+List, +DefNS, +TagOrBody, +Out)
+       
+%	save_attributes(+List, +DefNS, +TagOrBody, +Out, +NodeIDs, +Indent, +Anon)
 %
 %	Save a list of attributes.
 
-save_attributes2([], _, _, _, _, _).
-save_attributes2([H|T], DefNS, Where, Out, Indent, DB) :-
-	save_attribute(Where, H, DefNS, Out, Indent, DB),
-	save_attributes2(T, DefNS, Where, Out, Indent, DB).
+save_attributes2([], _, _, _, _, _, _).
+save_attributes2([H|T], DefNS, Where, Out, NodeIDs, Indent, Anon) :-
+	save_attribute(Where, H, DefNS, Out, NodeIDs, Indent, Anon),
+	save_attributes2(T, DefNS, Where, Out, NodeIDs, Indent, Anon).
 
-save_attribute(tag, rdf(_, Name, literal(Value)), DefNS, Out, Indent, _Anon) :-
+%%	save_attribute(+Where, +Triple, +DefNS, +Out, +NodeIDs, +Indent, +Anon)
+
+save_attribute(tag, rdf(_, Name, literal(Value)), DefNS, Out, _, Indent, _Anon) :-
 	AttIndent is Indent + 2,
 	rdf_att_id(Name, DefNS, NameText),
 	stream_property(Out, encoding(Encoding)),
 	xml_quote_attribute(Value, QVal, Encoding),
-	format(Out, '~N~*|~w="~w"', [AttIndent, NameText, QVal]).
-save_attribute(body, rdf(_,Name,literal(Literal)), DefNS, Out, Indent, _) :- !,
-	rdf_id(Name, DefNS, NameText),
+	format(Out, '~N~*|', [AttIndent]),
+	rdf_write_id(Out, NameText),
+	format(Out, '="~w"', [QVal]).
+save_attribute(body, rdf(_,Name,literal(Literal)), DefNS, Out, _, Indent, _) :- !,
+	rdf_p_id(Name, DefNS, NameText),
+	format(Out, '~N~*|<', [Indent]),
+	rdf_write_id(Out, NameText),
 	(   Literal = lang(Lang, Value)
 	->  rdf_id(Lang, DefNS, LangText),
-	    format(Out, '~N~*|<~w xml:lang="~w">',
-		   [Indent, NameText, LangText])
+	    format(Out, ' xml:lang="~w">', [LangText])
 	;   Literal = type(Type, Value)
 	->  (   rdf_equal(Type, rdf:'XMLLiteral')
-	    ->	format(Out, '~N~*|<~w rdf:parseType="Literal">',
-		       [Indent, NameText]),
+	    ->	write(Out, ' rdf:parseType="Literal">'),
 		Value = Literal
 	    ;	stream_property(Out, encoding(Encoding)),
 		rdf_value(Type, QVal, Encoding),
-		format(Out, '~N~*|<~w rdf:datatype="~w">',
-		       [Indent, NameText, QVal])
+		format(Out, ' rdf:datatype="~w">', [QVal])
 	    )
 	;   atomic(Literal)
-	->  format(Out, '~N~*|<~w>', [Indent, NameText]),
+	->  write(Out, '>'),
 	    Value = Literal
-	;   format(Out, '~N~*|<~w rdf:parseType="Literal">',
-		   [Indent, NameText]),
+	;   write(Out, ' rdf:parseType="Literal">'),
 	    Value = Literal
 	),
 	save_attribute_value(Value, Out, Indent),
-	format(Out, '</~w>', [NameText]).
-save_attribute(body, rdf(_, Name, Value), DefNS, Out, Indent, Anon) :-
-	anonymous_subject(Value),
-	memberchk(anon(Value, true, ValueTriples), Anon), !,
-	rdf_id(Name, DefNS, NameText),
-	SubIndent is Indent + 2,
-	(   rdf_equal(RdfType, rdf:type),
-	    rdf_equal(ListClass, rdf:'List'),
-	    memberchk(rdf(_, RdfType, ListClass), ValueTriples)
-	->  format(Out, '~N~*|<~w rdf:parseType="Collection">~n',
-		   [Indent, NameText]),
-	    rdf_save_list(ValueTriples, Out, Value, DefNS, SubIndent, Anon)
-	;   format(Out, '~N~*|<~w>~n',
-		   [Indent, NameText]),
-	    rdf_write_subject(ValueTriples, Out, Value, DefNS, SubIndent, Anon)
-	),
-	format(Out, '~N~*|</~w>~n', [Indent, NameText]).
-save_attribute(body, rdf(_, Name, Value), DefNS, Out, Indent, _Anon) :-
+	write(Out, '</'), rdf_write_id(Out, NameText), write(Out, '>').
+save_attribute(body, rdf(_, Name, Value), DefNS, Out, NodeIDs, Indent, Anon) :-
+	rdf_is_bnode(Value),
+	memberchk(anon(Value, Done, ValueTriples), Anon), !,
+	rdf_p_id(Name, DefNS, NameText),
+	format(Out, '~N~*|<', [Indent]),
+	rdf_write_id(Out, NameText),
+	(   var(Done)
+	->  Done = true,
+	    SubIndent is Indent + 2,
+	    (   rdf_equal(RdfType, rdf:type),
+		rdf_equal(ListClass, rdf:'List'),
+		memberchk(rdf(_, RdfType, ListClass), ValueTriples)
+	    ->  format(Out, ' rdf:parseType="Collection">~n', []),
+		rdf_save_list(ValueTriples, Out, Value, NodeIDs, DefNS, SubIndent, Anon)
+	    ;   format(Out, '>~n', []),
+		rdf_write_subject(ValueTriples, Out, Value, NodeIDs, DefNS, SubIndent, Anon)
+	    ),
+	    format(Out, '~N~*|</', [Indent]),
+	    rdf_write_id(Out, NameText),
+	    format(Out, '>~n', [])
+	;   get_assoc(Value, NodeIDs, NodeID)
+	->  format(Out, ' rdf:nodeID="~w"/>', [NodeID])
+	;   assertion(fail)
+	).
+save_attribute(body, rdf(_, Name, Value), DefNS, Out, _, Indent, _Anon) :-
 	stream_property(Out, encoding(Encoding)),
 	rdf_value(Value, QVal, Encoding),
-	rdf_id(Name, DefNS, NameText),
-	format(Out, '~N~*|<~w rdf:resource="~w"/>', [Indent, NameText, QVal]).
+	rdf_p_id(Name, DefNS, NameText),
+	format(Out, '~N~*|<', [Indent]),
+	rdf_write_id(Out, NameText),
+	format(Out, ' rdf:resource="~w"/>', [QVal]).
 
 save_attribute_value(Value, Out, _) :-	% strings
 	atom(Value), !,
@@ -425,15 +517,15 @@ save_attribute_value(Value, Out, Indent) :-
 save_attribute_value(Value, _Out, _) :-
 	throw(error(save_attribute_value(Value), _)).
 
-rdf_save_list(_, _, List, _, _, _) :-
+rdf_save_list(_, _, List, _, _, _, _) :-
 	rdf_equal(List, rdf:nil), !.
-rdf_save_list(ListTriples, Out, List, DefNS, Indent, Anon) :-
+rdf_save_list(ListTriples, Out, List, NodeIDs, DefNS, Indent, Anon) :-
 	rdf_equal(RdfFirst, rdf:first),
 	memberchk(rdf(List, RdfFirst, First), ListTriples),
-	(   anonymous_subject(First),
+	(   rdf_is_bnode(First),
 	    memberchk(anon(First, true, FirstTriples), Anon)
 	->  nl(Out),
-	    rdf_write_subject(FirstTriples, Out, First, DefNS, Indent, Anon)
+	    rdf_write_subject(FirstTriples, Out, First, NodeIDs, DefNS, Indent, Anon)
 	;   stream_property(Out, encoding(Encoding)),
 	    rdf_value(First, QVal, Encoding),
 	    format(Out, '~N~*|<rdf:Description about="~w"/>',
@@ -443,22 +535,32 @@ rdf_save_list(ListTriples, Out, List, DefNS, Indent, Anon) :-
 	    memberchk(rdf(List, RdfRest, List2), ListTriples),
 	    \+ rdf_equal(List2, rdf:nil),
 	    memberchk(anon(List2, true, List2Triples), Anon)
-	->  rdf_save_list(List2Triples, Out, List2, DefNS, Indent, Anon)
+	->  rdf_save_list(List2Triples, Out, List2, NodeIDs, DefNS, Indent, Anon)
 	;   true
 	).
 
-%%	anonymous_subject(+Subject)
+%%	rdf_p_id(+Resource, +DefNS, -NSLocal)
+%
+%	As rdf_id/3 for predicate names.  Maps _:<N> to rdf:li.
 %	
-%	Test if a resource is anonymous. This is highly dubious.
-%	Probably we need to store this in the database.  The current
-%	release of the RDF parser guarantees that all anonymous ids
-%	start with __.
+%	@tbd	Ensure we are talking about an rdf:Bag
 
-anonymous_subject(S) :-
-	atom(S),
-	sub_atom(S, 0, _, _, '__'), !.
+rdf_p_id(LI, _, 'rdf:li') :-
+	is_bag_li_predicate(LI), !.
+rdf_p_id(Resource, DefNS, NSLocal) :-
+	rdf_id(Resource, DefNS, NSLocal).
 
-%	rdf_id(+Resource, +DefNS, -NSLocal)
+%%	is_bag_li_predicate(+Pred) is semidet.
+%
+%	True if Pred is _:N, as used  for members of an rdf:Bag, rdf:Seq
+%	or rdf:Alt.
+
+is_bag_li_predicate(Pred) :-
+	atom_concat('_:', AN, Pred),
+	catch(atom_number(AN, N), _, true), integer(N), N >= 0, !.
+
+
+%%	rdf_id(+Resource, +DefNS, -NSLocal)
 %	
 %	Generate a NS:Local name for Resource given the indicated
 %	default namespace.  This call is used for elements.
@@ -474,6 +576,17 @@ rdf_id(Id, _, NS:Local) :-
 rdf_id(Id, _, Id).
 
 
+%%	rdf_write_id(+Out, +NSLocal) is det.
+%
+%	Write an identifier. We cannot use native write on it as both NS
+%	and Local can be operators.
+
+rdf_write_id(Out, NS:Local) :- !,
+	format(Out, '~w:~w', [NS, Local]).
+rdf_write_id(Out, Atom) :-
+	write(Out, Atom).
+
+
 rdf_att_id(Id, _, NS:Local) :-
 	ns(NS, Full),
 	Full \== '',
@@ -495,7 +608,7 @@ rdf_att_id(Id, _, Id).
 
 rdf_value(V, Text, Encoding) :-
 	to_be_described(Prefix),
-	atom_concat(Prefix, V1, V), !,
+	atom_concat(Prefix, V1, V),
 	ns(NS, Full),
 	atom_concat(Full, Local, V1), !,
 	rdf_quote_uri(Local, QLocal0),
