@@ -1,4 +1,4 @@
-/*  $Id: table.c,v 1.11 2003/10/02 08:02:27 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -28,6 +28,8 @@
 
 #define O_ORDER				/* include order.c package */
 
+#include <SWI-Stream.h>
+
 #include "table.h"
 #include "error.h"
 #include <stdlib.h>
@@ -55,8 +57,6 @@
 #include <floatingpoint.h>		/* strtod() prototype */
 #endif
 
-#include <SWI-Stream.h>
-
 #ifdef O_DEBUG
 #define DEBUG(g) g
 #else
@@ -89,6 +89,7 @@ open_table(+File, +Fields, +Options, -Handle)
 		string		read field as SWI-Prolog string
 		code_list	read field as list of ASCII codes
 		integer		read field as integer
+		hexadecimal	read field as hex integer
 		float		read field as floating point number
 
 	Options is a list of options.  Each option is of the form
@@ -146,6 +147,7 @@ static atom_t ATOM_file;
 static atom_t ATOM_float;
 static atom_t ATOM_functor;
 static atom_t ATOM_integer;
+static atom_t ATOM_hexadecimal;
 static atom_t ATOM_key_field;
 static atom_t ATOM_map_space_to_underscore;
 static atom_t ATOM_prefix;
@@ -181,6 +183,7 @@ init_constants()
   ATOM_float			= PL_new_atom("float");
   ATOM_functor			= PL_new_atom("functor");
   ATOM_integer			= PL_new_atom("integer");
+  ATOM_hexadecimal		= PL_new_atom("hexadecimal");
   ATOM_key_field		= PL_new_atom("key_field");
   ATOM_map_space_to_underscore	= PL_new_atom("map_space_to_underscore");
   ATOM_prefix			= PL_new_atom("prefix");
@@ -200,25 +203,116 @@ init_constants()
 }
 
 
+		 /*******************************
+		 *	       ERRORS		*
+		 *******************************/
+
+static int
+domain_error(term_t actual, const char *expected)
+{ term_t ex = PL_new_term_ref();
+
+  PL_unify_term(ex, PL_FUNCTOR_CHARS, "error", 2,
+		      PL_FUNCTOR_CHARS, "domain_error", 2,
+		        PL_CHARS, expected,
+		        PL_TERM, actual,
+		      PL_VARIABLE);
+
+  return PL_raise_exception(ex);
+}
+
+
+static int
+type_error(term_t actual, const char *expected)
+{ term_t ex = PL_new_term_ref();
+
+  PL_unify_term(ex, PL_FUNCTOR_CHARS, "error", 2,
+		      PL_FUNCTOR_CHARS, "type_error", 2,
+		        PL_CHARS, expected,
+		        PL_TERM, actual,
+		      PL_VARIABLE);
+
+  return PL_raise_exception(ex);
+}
+
+
+static int
+existence_error(term_t actual, const char *expected)
+{ term_t ex = PL_new_term_ref();
+
+  PL_unify_term(ex, PL_FUNCTOR_CHARS, "error", 2,
+		      PL_FUNCTOR_CHARS, "existence_error", 2,
+		        PL_CHARS, expected,
+		        PL_TERM, actual,
+		      PL_VARIABLE);
+
+  return PL_raise_exception(ex);
+}
+
+
+static int
+format_error(const char *pred, size_t pos, Field f)
+{ char buf[1024];
+
+  sprintf(buf, "%s: bad record, field %d (%s), char-index %ld",
+	  pred, f->index, PL_atom_chars(f->name), (long)pos);
+
+  return PL_warning(buf);
+}
+
+
 
 		 /*******************************
 		 *	 HIGH-LEVEL GET-*	*
 		 *******************************/
 
 static int
-get_table(term_t handle, Table *table)
-{ long l;
+get_size_ex(term_t t, size_t *v)
+{ int64_t i;
 
-  if ( PL_get_long(handle, &l) )
-  { Table t = (Table)l;
+  if ( PL_get_int64(t, &i) )
+  { if ( i < 0 )
+      return domain_error(t, "nonneg");
+      
+    *v = (size_t)i;			/* TBD: Check on 32-bit systems */
+    return TRUE;
+  }
+
+  return type_error(t, "integer");
+}
+
+
+static int
+get_offset_ex(term_t t, table_offset_t *v)
+{ int64_t i;
+
+  if ( PL_get_int64(t, &i) )
+  { if ( i < 0 )
+      return domain_error(t, "nonneg");
+      
+    *v = (table_offset_t)i;		/* TBD: Check on 32-bit systems */
+    return TRUE;
+  }
+
+  return type_error(t, "integer");
+}
+
+
+static int
+get_table_ex(term_t handle, Table *table)
+{ int64_t l;
+
+  if ( PL_get_int64(handle, &l) )
+  { Table t = (Table)(intptr_t)l;
 
     if ( t->magic == TABLE_MAGIC )
     { *table = t;
       return TRUE;
     }
+
+    return existence_error(handle, "table");
   }
     
-  return FALSE;
+  return type_error(handle, "table");
 }
 
 
@@ -232,6 +326,8 @@ get_type(atom_t typename, int *type)
     *type = FIELD_CODELIST;
   else if ( typename == ATOM_integer )
     *type = FIELD_INTEGER;
+  else if ( typename == ATOM_hexadecimal )
+    *type = FIELD_HEX;
   else if ( typename == ATOM_float )
     *type = FIELD_FLOAT;
   else
@@ -323,7 +419,9 @@ pl_new_table(term_t file, term_t columns, term_t options, term_t handle)
   table->escape_table = NULL;
 
   if ( !PL_get_atom(file, &table->file) )
+  { free(table);
     return error(ERR_INSTANTIATION, "open_table/4", 1, file);
+  }
 
   while(PL_get_list(tail, head, tail))
   { int arity;
@@ -334,7 +432,9 @@ pl_new_table(term_t file, term_t columns, term_t options, term_t handle)
 	 !PL_get_arg(1, head, arg) ||
 	 !PL_get_atom(arg, &typename) ||
 	 !get_type(typename, &fields[nfields].type) )
+    { free(table);
       return error(ERR_INSTANTIATION, "new_table/4", 2, columns);
+    }
 
     fields[nfields].index = nfields;	/* index number (0..) */
     fields[nfields].width = 0;		/* variable-width field */
@@ -365,12 +465,15 @@ pl_new_table(term_t file, term_t columns, term_t options, term_t handle)
 	  }
 	} else
 	{ colerr:
+	  free(table);
 	  return error(ERR_INSTANTIATION, "new_table/4", 2, columns);
 	}
       }
       if ( !PL_get_nil(tail2) )
+      { free(table);
 	return error(ERR_INSTANTIATION, "new_table/4", 2, columns);
-      
+      }
+
       if ( fields[nfields].flags & FIELD_SORTED )
 	table->keyfield = nfields;
 
@@ -383,7 +486,9 @@ pl_new_table(term_t file, term_t columns, term_t options, term_t handle)
     nfields++;
   }
   if ( !PL_get_nil(tail) )
+  { free(table);
     return error(ERR_INSTANTIATION, "new_table/4", 2, columns);
+  }
 
   table->record_sep = '\n';
   table->field_sep  = ' ';
@@ -444,6 +549,7 @@ pl_new_table(term_t file, term_t columns, term_t options, term_t handle)
   }
   if ( !PL_get_nil(tail) )
   { err3:
+    free(table);
     return error(ERR_INSTANTIATION, "new_table/4", 3, options);
   }
 
@@ -465,7 +571,7 @@ pl_new_table(term_t file, term_t columns, term_t options, term_t handle)
   table->size   = -1;
   table->buffer = NULL;
   table->window = NULL;
-#ifdef WIN32
+#ifdef __WINDOWS__
   table->hfile  = NULL;
   table->hmap   = NULL;
 #endif
@@ -481,7 +587,7 @@ static int
 open_table(Table table)
 { if ( !table->buffer )
   {
-#ifdef WIN32
+#ifdef __WINDOWS__
     BY_HANDLE_FILE_INFORMATION info;
 
     table->hfile = CreateFile(PL_atom_chars(table->file),
@@ -503,7 +609,7 @@ open_table(Table table)
 				    NULL,
 				    PAGE_READONLY,
 				    0L,
-				    table->size,
+				    (DWORD)table->size, /* Truncated? */
 				    NULL);
     if ( !table->hmap )
       goto errio;
@@ -536,7 +642,7 @@ open_table(Table table)
 
     return error(ERR_IO, "open_table/1", id, NULL);
   }
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 #ifdef __unix__
     struct stat buf;
@@ -585,15 +691,15 @@ open_table(Table table)
 static foreign_t
 pl_table_window(term_t handle, term_t start, term_t size)
 { Table table;
-  long from;
-  long wsize;
+  size_t from;
+  size_t wsize;
 
-  if ( !get_table(handle, &table) )
-    return error(ERR_INSTANTIATION, "table_window/3", 1, handle);
-  if ( !PL_get_long(start, &from) || from < 0 )
-    return error(ERR_INSTANTIATION, "table_window/3", 2, start);
-  if ( !PL_get_long(size, &wsize) || wsize < 0 )
-    return error(ERR_INSTANTIATION, "table_window/3", 3, size);
+  if ( !get_table_ex(handle, &table) )
+    return FALSE;
+  if ( !get_size_ex(start, &from) )
+    return FALSE;
+  if ( !get_size_ex(size, &wsize) )
+    return FALSE;
 
   if ( from > table->size )
     from = table->size;
@@ -613,8 +719,8 @@ pl_get_table_attribute(term_t handle, term_t name, term_t value)
   atom_t n;
   int arity;
 
-  if ( !get_table(handle, &table) )
-    return error(ERR_INSTANTIATION, "get_table_attribute/3", 1, handle);
+  if ( !get_table_ex(handle, &table) )
+    return FALSE;
 
   
   if ( PL_get_name_arity(name, &n, &arity) )
@@ -687,6 +793,9 @@ unify_field_info(term_t t, Field field)	/* name(Type, Flags) */
     case FIELD_INTEGER:
       type = ATOM_integer;
       break;
+    case FIELD_HEX:
+      type = ATOM_hexadecimal;
+      break;
     case FIELD_FLOAT:
       type = ATOM_float;
       break;
@@ -751,8 +860,8 @@ static foreign_t
 pl_open_table(term_t handle)
 { Table table;
 
-  if ( !get_table(handle, &table) )
-    return error(ERR_INSTANTIATION, "open_table/1", 1, handle);
+  if ( !get_table_ex(handle, &table) )
+    return FALSE;
 
   return open_table(table);
 }
@@ -762,12 +871,12 @@ static foreign_t
 pl_close_table(term_t handle)
 { Table table;
 
-  if ( !get_table(handle, &table) )
-    return error(ERR_INSTANTIATION, "close_table/1", 1, handle);
+  if ( !get_table_ex(handle, &table) )
+    return FALSE;
 
   if ( table->buffer )
   {
-#ifdef WIN32
+#ifdef __WINDOWS__
     if ( table->buffer )
       UnmapViewOfFile(table->buffer);
     if ( table->hmap )
@@ -802,12 +911,14 @@ pl_free_table(term_t handle)
   if ( !pl_close_table(handle) )
     PL_fail;
 
-  get_table(handle, &table);
+  if ( !get_table_ex(handle, &table) )
+    return FALSE;
   
   table->magic = 0;			/* so it won't be recognised */
   if ( table->escape_table )
     free(table->escape_table);
 
+  free(table->fields);
   free(table);
 
   PL_succeed;
@@ -853,10 +964,10 @@ find_start_of_record(Table t, table_offset_t start)
 { char *s;
   int er = t->record_sep;
 
-  if ( start < 0 || start > t->window_size )
+  if ( start < 0 || start > (table_offset_t)t->window_size )
     return -1;
 
-  if ( start == t->window_size && start > 0 )
+  if ( start == (table_offset_t)t->window_size && start > 0 )
     start--;
 
   s = t->window + start;
@@ -882,7 +993,7 @@ previous_record(Table t, table_offset_t start)
 { char *s;
   int er = t->record_sep;
     
-  if ( start < 0 || start > t->window_size )
+  if ( start < 0 || start > (table_offset_t)t->window_size )
     return -1;
 
   s = t->window + start - 1;
@@ -899,10 +1010,10 @@ pl_previous_record(term_t handle, term_t here, term_t prev)
 { Table t;
   table_offset_t start;
 
-  if ( !get_table(handle, &t) )
-    return error(ERR_INSTANTIATION, "previous_record/3", 1, handle);
-  if ( !PL_get_long(here, &start) )
-    return error(ERR_INSTANTIATION, "previous_record/3", 2, here);
+  if ( !get_table_ex(handle, &t) )
+    return FALSE;
+  if ( !get_offset_ex(here, &start) )
+    return FALSE;
 
   if ( !open_table(t) )
     PL_fail;
@@ -929,15 +1040,16 @@ pl_start_of_record(term_t handle,		/* table */
 		   term_t recstart,		/* return */
 		   control_t control)		/* backtracking control */
 { Table table;
-  table_offset_t n, f, t;
+  size_t n, f;
+  table_offset_t t;
   char *end;				/* pointer to end of search */
   char *start;				/* start of search */
   int er;
 
   switch(PL_foreign_control(control))
   { case PL_FIRST_CALL:
-      if ( !PL_get_long(from, &f) )
-	return error(ERR_INSTANTIATION, "start_of_record/4", 2, from);
+      if ( !get_size_ex(from, &f) )
+	return FALSE;
       break;
     case PL_REDO:
       f = PL_foreign_context(control);
@@ -947,15 +1059,15 @@ pl_start_of_record(term_t handle,		/* table */
       PL_succeed;
   }
 
-  if ( !get_table(handle, &table) )
-    return error(ERR_INSTANTIATION, "start_of_record/4", 1, handle);
-  if ( !PL_get_long(to, &t) )
-    return error(ERR_INSTANTIATION, "start_of_record/4", 3, t);
+  if ( !get_table_ex(handle, &table) )
+    return FALSE;
+  if ( !get_offset_ex(to, &t) )
+    return FALSE;
 
   if ( !open_table(table) )
     return FALSE;
 					/* find end of search */
-  if ( t < 0 || t > table->window_size )
+  if ( t < 0 || (size_t)t > table->window_size )
     end = table->window + table->window_size;
   else
     end = table->window + t;
@@ -1047,18 +1159,24 @@ out:
 }
 
 
-int
-digitval(int chr)
+static int
+digitval(int chr, int base)
 { if ( chr >= '0' && chr <= '9' )
     return chr - '0';
+  if ( base == 16 )
+  { if ( chr >= 'a' && chr <= 'f' )
+      return chr + 10 - 'a';
+    if ( chr >= 'A' && chr <= 'F' )
+      return chr + 10 - 'A';
+  }
 
   return -1;
 }
 
 
 static void
-tab_memcpy(Table table, int flags, char *to, const char *from, int len)
-{ int i = len;
+tab_memcpy(Table table, int flags, char *to, const char *from, size_t len)
+{ size_t i = len;
   char *t = to;
 
   if ( flags & FIELD_DOWNCASE )
@@ -1103,7 +1221,7 @@ tab_memcpy(Table table, int flags, char *to, const char *from, int len)
 
 static int
 unify_field_text(Table t, int flags, int type,
-		 term_t arg, const char *s, int len)
+		 term_t arg, const char *s, size_t len)
 { char *tmp;
   int rval = FALSE;
 #ifndef HAVE_ALLOCA
@@ -1164,24 +1282,31 @@ read_field(Table t, Field f, table_offset_t start, table_offset_t *end, term_t a
     case FIELD_STRING:
     case FIELD_CODELIST:
       return unify_field_text(t, f->flags, type, arg, s, z-s);
+    { int base;
+    case FIELD_HEX:
+      base = 16;
+      goto case_int;
     case FIELD_INTEGER:
-    { long l = 0;
-      char *a = s;
-      int digits=0;
+    { long l;
+      char *a;
+      int digits;
 
-      for(; a<z; a++)
+      base=10;
+    case_int:
+
+      for(l=0,digits=0,a=s; a<z; a++)
       { int code;
 
-	if ( (code = digitval(*a)) >= 0 )
+	if ( (code = digitval(*a, base)) >= 0 )
 	{ digits++;
-	  l = l*10+code;
+	  l = l*base+code;
 	}
 	else if ( !isBlank(*a) )
 	{ if ( f->flags & FIELD_ALLOWBADNUM )
 	  { type = FIELD_ATOM;
 	    goto case_atom;
 	  }
-	  return error(ERR_FORMAT, "read_record", s-t->window, f);
+	  return format_error("read_record", s-t->window, f);
 	}
       }
 
@@ -1190,11 +1315,12 @@ read_field(Table t, Field f, table_offset_t start, table_offset_t *end, term_t a
 	{ type = FIELD_ATOM;
 	  goto case_atom;
 	}
-	return error(ERR_FORMAT, "read_record", s-t->window, f);
+	return format_error("read_record", s-t->window, f);
       }
 
       return PL_unify_integer(arg, l);
     }
+    }
     case FIELD_FLOAT:
     { char *e;
       double g;
@@ -1208,7 +1334,7 @@ read_field(Table t, Field f, table_offset_t start, table_offset_t *end, term_t a
       { type = FIELD_ATOM;
 	goto case_atom;
       } else
-	return error(ERR_FORMAT, "read_record", s-t->window, f);
+	return format_error("read_record", s-t->window, f);
     }
   }
 
@@ -1253,10 +1379,10 @@ pl_read_record(term_t handle, term_t from, term_t to, term_t record)
 { Table table;
   table_offset_t start, end;
 
-  if ( !get_table(handle, &table) )
-    return error(ERR_INSTANTIATION, "read_record/4", 1, handle);
-  if ( !PL_get_long(from, &start) )
-    return error(ERR_INSTANTIATION, "read_record/4", 2, from);
+  if ( !get_table_ex(handle, &table) )
+    return FALSE;
+  if ( !get_offset_ex(from, &start) )
+    return FALSE;
 
   if ( !open_table(table) )
     return FALSE;
@@ -1275,12 +1401,12 @@ foreign_t
 pl_read_record_data(term_t handle, term_t from, term_t to, term_t record)
 { Table table;
   table_offset_t start, end;
-  int len;
+  size_t len;
 
-  if ( !get_table(handle, &table) )
-    return error(ERR_INSTANTIATION, "read_record/4", 1, handle);
-  if ( !PL_get_long(from, &start) )
-    return error(ERR_INSTANTIATION, "read_record/4", 2, from);
+  if ( !get_table_ex(handle, &table) )
+    return FALSE;
+  if ( !get_offset_ex(from, &start) )
+    return FALSE;
 
   if ( !open_table(table) )
     return FALSE;
@@ -1312,10 +1438,10 @@ pl_read_fields(term_t handle, term_t from, term_t to, term_t fields)
   term_t argv[MAXFIELDS*sizeof(term_t)];
 #endif
 
-  if ( !get_table(handle, &table) )
-    return error(ERR_INSTANTIATION, "read_fields/4", 1, handle);
-  if ( !PL_get_long(from, &start) )
-    return error(ERR_INSTANTIATION, "read_fields/4", 2, from);
+  if ( !get_table_ex(handle, &table) )
+    return FALSE;
+  if ( !get_offset_ex(from, &start) )
+    return FALSE;
 
   if ( !open_table(table) ||
        (start = find_start_of_record(table, start)) < 0 )
@@ -1401,7 +1527,7 @@ match_field(Table t, Field f, QueryField q, table_offset_t start, table_offset_t
   { case FIELD_ATOM:
     case FIELD_STRING:
     case FIELD_CODELIST:
-    { int len = z-a;
+    { size_t len = z-a;
 #ifdef HAVE_ALLOCA
       char *tmp = alloca(len+1);
 #define DEALLOC()
@@ -1461,7 +1587,7 @@ match_field(Table t, Field f, QueryField q, table_offset_t start, table_offset_t
 	}
       } else if ( q->flags & QUERY_SUBSTRING ) /* Use Boyle Moore */
       { if ( q->ord )
-	{ int ls = q->length;
+	{ size_t ls = q->length;
 	  int i;
 
 	  for(i=0; i+ls<=len; i++)
@@ -1473,8 +1599,8 @@ match_field(Table t, Field f, QueryField q, table_offset_t start, table_offset_t
           DEALLOC();
 	  return MATCH_NE;
 	} else
-	{ int ls = q->length;
-	  int i;
+	{ size_t ls = q->length;
+	  size_t i;
 
 	  for(i=0; i+ls<=len; i++)
 	  { if ( strncmp(q->value.s, &tmp[i], ls) == 0 )
@@ -1488,21 +1614,28 @@ match_field(Table t, Field f, QueryField q, table_offset_t start, table_offset_t
       }
     }
 
+    { int base;
+    case FIELD_HEX:
+      base = 16;
+      goto case_int;
     case FIELD_INTEGER:
-    { long l = 0;
-      int digits=0;
+    { long l;
+      int digits;
 
-      for(; a<z; a++)
+      base = 10;
+    case_int:
+
+      for(l=0, digits=0; a<z; a++)
       { int code;
 
-	if ( (code = digitval(*a)) >= 0 )
+	if ( (code = digitval(*a, base)) >= 0 )
 	{ digits++;
-	  l = l*10+code;
+	  l = l*base+code;
 	}
 	else if ( !isBlank(*a) )
 	{ if ( f->flags & FIELD_ALLOWBADNUM )
 	    return MATCH_NE;
-	  error(ERR_FORMAT, "match_record", a-t->window, f);
+	  format_error("match_record", a-t->window, f);
 	  return MATCH_NORECORD;
 	}
       }
@@ -1510,7 +1643,7 @@ match_field(Table t, Field f, QueryField q, table_offset_t start, table_offset_t
       if ( !digits )
       { if ( f->flags & FIELD_ALLOWBADNUM )
 	  return MATCH_NE;
-	error(ERR_FORMAT, "match_record", a-t->window, f);
+	format_error("match_record", a-t->window, f);
       }
 
       if ( q->flags & QUERY_READ )
@@ -1520,6 +1653,7 @@ match_field(Table t, Field f, QueryField q, table_offset_t start, table_offset_t
 
       return q->value.i == l ? MATCH_EQ : q->value.i < l ? MATCH_LT : MATCH_GT;
     }
+    }
 
     case FIELD_FLOAT:
     { char *e;
@@ -1531,7 +1665,7 @@ match_field(Table t, Field f, QueryField q, table_offset_t start, table_offset_t
       if ( z != e )
       { if ( f->flags & FIELD_ALLOWBADNUM )
 	  return MATCH_NE;
-	error(ERR_FORMAT, "match_record", a-t->window, f);
+	format_error("match_record", a-t->window, f);
 	return MATCH_NORECORD;
       }
 
@@ -1604,7 +1738,7 @@ execute_binary_search(Query q)
 
     switch( match_record(q, here, &next, MR_KEY_ONLY) )
     { case MATCH_NORECORD:
-      { if ( here >= t->window_size )
+      { if ( here >= (table_offset_t)t->window_size )
 	  return FALSE;
 	here = next;
 	continue;
@@ -1645,7 +1779,7 @@ execute_binary_search(Query q)
 
   next:
     if ( low == here )
-    { while(here <= high && here < t->window_size)
+    { while(here <= high && here < (table_offset_t)t->window_size)
       { switch( match_record(q, here, &next, MR_KEY_ONLY) )
 	{ case MATCH_EQ:
 	    return here;
@@ -1762,6 +1896,7 @@ make_query(Table t, term_t from)
 	    }
 	    qf->length = strlen(qf->value.s);
 	    break;
+	  case FIELD_HEX:
 	  case FIELD_INTEGER:
 	    if ( !PL_get_long(arg, &qf->value.i) )
 	      goto err2;
@@ -1853,8 +1988,8 @@ pl_in_table(term_t handle, term_t spec, term_t record, control_t control)
   { case PL_FIRST_CALL:
     { Table t;
       
-      if ( !get_table(handle, &t) )
-	return error(ERR_INSTANTIATION, "in_table/3", 1, handle);
+      if ( !get_table_ex(handle, &t) )
+	return FALSE;
       if ( !open_table(t) )
 	PL_fail;
       if ( !(q=make_query(t, spec)) )
@@ -1927,7 +2062,7 @@ pl_in_table(term_t handle, term_t spec, term_t record, control_t control)
   } 
 
 
-  while(q->offset < q->table->window_size)
+  while(q->offset < (table_offset_t)q->table->window_size)
   { table_offset_t next;
 
     if ( match_record(q, q->offset, &next, 0) == MATCH_EQ )
