@@ -3,9 +3,9 @@
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2008, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -34,6 +34,7 @@
 :- use_module(library(pce)).
 :- use_module(library(emacs_extend)).
 :- use_module(library(pce_prolog_xref)).
+:- use_module(library(prolog_source)).
 :- use_module(library(lists)).
 :- use_module(library(operators)).
 :- use_module(library(debug)).
@@ -275,7 +276,7 @@ process_use_module(File, Src) :-
 
 colourise(TB, Fd) :-
 	character_count(Fd, Pos),
-	emacs_prolog_mode:read_term_from_stream(Fd, Pos,
+	emacs_prolog_mode:read_term_from_stream(TB, Fd, Pos,
 						Term,
 						Error,
 						_Singletons,
@@ -481,6 +482,10 @@ colourise_dcg_subgoals([Pos|T], N, Body, Origin, TB) :-
 	NN is N + 1,
 	colourise_dcg_subgoals(T, NN, Body, Origin, TB).
 
+dcg_extend(Term, _) :-
+	var(Term), !, fail.
+dcg_extend(M:Term, M:Goal) :-
+	dcg_extend(Term, Goal).
 dcg_extend(Term, Goal) :-
 	callable(Term),
 	Term =.. List,
@@ -591,6 +596,9 @@ instantiate_meta([H|T]) :-
 
 expand_meta(MetaSpec, Goal, Goal) :-
 	MetaSpec == 0.
+expand_meta(MetaSpec, M:Goal, M:Expanded) :-
+	atom(M), !,
+	expand_meta(MetaSpec, Goal, Expanded).
 expand_meta(MetaSpec, Goal, Expanded) :-
 	integer(MetaSpec),
 	callable(Goal), !,
@@ -733,11 +741,11 @@ colourise_exports2(_, _, _).
 %	Colourise import list from use_module/2, importing from File.
 
 colourise_imports(List, File, TB, Pos) :-
-	(   xref_public_list(File, _, Public, TB)
+	(   xref_public_list(File, Path, Public, TB)
 	->  true
 	;   Public = []
 	),
-	colourise_imports(List, File, Public, TB, Pos).
+	colourise_imports(List, Path, Public, TB, Pos).
 
 colourise_imports([], _, _, _, _).
 colourise_imports(List, File, Public, TB, list_position(_,_,ElmPos,Tail)) :- !,
@@ -746,17 +754,34 @@ colourise_imports(List, File, Public, TB, list_position(_,_,ElmPos,Tail)) :- !,
 	;   colour_item(type_error(list), TB, Tail)
 	),
 	colourise_imports2(List, File, Public, TB, ElmPos).
+colourise_imports(except(Except), File, Public, TB,
+		  term_position(_,_,FF,FT,[LP])) :- !,
+	colour_item(keyword(except), TB, FF-FT),
+	colourise_imports(Except, File, Public, TB, LP).
 colourise_imports(_, _, _, TB, Pos) :-
 	colour_item(type_error(list), TB, Pos).	
 
 colourise_imports2([G0|GT], File, Public, TB, [P0|PT]) :- !,
-	colourise_declaration(G0, TB, P0),
+	colourise_import(G0, File, TB, P0),
 	colourise_imports2(GT, File, Public, TB, PT).
 colourise_imports2(_, _, _, _, _).
 
+
+colourise_import(PI as Name, File, TB, term_position(_,_,FF,FT,[PP,NP])) :-
+	pi_to_term(PI, Goal), !,
+	colour_item(goal(imported(File), Goal), TB, PP),
+	functor(Goal, _, Arity),
+	functor(NewGoal, Name, Arity),
+	goal_classification(TB, NewGoal, [], Class),
+	colour_item(goal(Class, NewGoal), TB, NP),
+	colour_item(keyword(as), TB, FF-FT).
+colourise_import(PI, _, TB, Pos) :-
+	colourise_declaration(PI, TB, Pos).
+
+
 %	colourise_declarations(+Term, +TB, +Pos)
 %	
-%	Colourise the Name/Arity lists of dynamic, multifile, etc
+%	Colourise the Predicate indicator lists of dynamic, multifile, etc
 %	declarations.
 
 colourise_declarations((Head,Tail), TB,
@@ -766,28 +791,27 @@ colourise_declarations((Head,Tail), TB,
 colourise_declarations(Last, TB, Pos) :-
 	colourise_declaration(Last, TB, Pos).
 
-colourise_declaration(Name/Arity, TB, Pos) :-
-	atom(Name), integer(Arity), !,
-	functor(Goal, Name, Arity),
+colourise_declaration(PI, TB, Pos) :-
+	pi_to_term(PI, Goal), !,
 	goal_classification(TB, Goal, [], Class),
 	colour_item(goal(Class, Goal), TB, Pos).
-colourise_declaration(Name//DCGArity, TB, Pos) :-
-	atom(Name), integer(DCGArity), !,
-	Arity is DCGArity+2,
-	functor(Goal, Name, Arity),
-	goal_classification(TB, Goal, [], Class),
-	colour_item(goal(Class, Goal), TB, Pos).
-colourise_declaration(Module:Name/Arity, TB,
+colourise_declaration(Module:PI, TB,
 		      term_position(_,_,_,_,[PM,PG])) :-
-	atom(Module), atom(Name), integer(Arity), !,
+	atom(Module), pi_to_term(PI, Goal), !,
 	colour_item(module(M), TB, PM),
-	functor(Goal, Name, Arity),
 	colour_item(goal(extern(M), Goal), TB, PG).
 colourise_declaration(op(_,_,_), TB, Pos) :-
 	colour_item(exported_operator, TB, Pos).
 colourise_declaration(_, TB, Pos) :-
 	colour_item(type_error(export_declaration), TB, Pos).
 
+pi_to_term(Name/Arity, Term) :-
+	atom(Name), integer(Arity), !,
+	functor(Term, Name, Arity).
+pi_to_term(Name//Arity0, Term) :-
+	atom(Name), integer(Arity0), !,
+	Arity is Arity0 + 2,
+	functor(Term, Name, Arity).
 
 %	colour_item(+Class, +TB, +Pos)
 %
@@ -878,7 +902,7 @@ goal_classification(Goal, built_in) :-
 	built_in_predicate(Goal), !.
 goal_classification(Goal, autoload) :-	% SWI-Prolog
 	functor(Goal, Name, Arity),
-	'$in_library'(Name, Arity), !.
+	'$in_library'(Name, Arity, _Path), !.
 goal_classification(Goal, global) :-	% SWI-Prolog
 	current_predicate(_, user:Goal), !.
 goal_classification(SS, expanded) :-	% XPCE (TBD)
@@ -918,6 +942,8 @@ built_in_predicate(endif).
 goal_colours(module(_,_),	     built_in-[identifier,exports]).
 goal_colours(use_module(_),	     built_in-[file]).
 goal_colours(use_module(File,_),     built_in-[file,imports(File)]).
+goal_colours(reexport(_),	     built_in-[file]).
+goal_colours(reexport(File,_),       built_in-[file,imports(File)]).
 goal_colours(dynamic(_),	     built_in-[predicates]).
 goal_colours(thread_local(_),	     built_in-[predicates]).
 goal_colours(multifile(_),	     built_in-[predicates]).
@@ -1016,7 +1042,7 @@ def_style(head(hook),	  	style(underline  := @on, colour := blue)).
 def_style(head(meta),	  	@default).
 def_style(head(constraint(_)),	style(bold := @on, colour := darkcyan)).
 def_style(head(_),	  	style(bold := @on)).
-
+def_style(module(_),		style(colour := dark_slate_blue)).
 def_style(comment,		style(colour := dark_green)).
 
 def_style(directive,	  	style(background := grey90)).
@@ -1042,6 +1068,7 @@ def_style(class(undefined,_),	style(colour := red,
 def_style(prolog_data,		style(colour := blue,
 				      underline  := @on)).
 
+def_style(keyword(_), 		style(colour := blue)).
 def_style(identifier, 		style(bold := @on)).
 def_style(delimiter,		style(bold := @on)).
 def_style(expanded,		style(colour := blue,
@@ -1504,10 +1531,20 @@ identify(F) :->
 identify_pred(Term, _, Summary) :-
 	identify(Term, String),
 	new(Summary, string(String)).
-identify_pred(built_in, F, Summary) :-	% SWI-Prolog documented built-in
-	get(F, predicate, Pred),
+identify_pred(Class, F, Summary) :-		% SWI-Prolog documented built-in
+	get(F, predicate, Pred),		% & PlDoc summaries
 	get(Pred, summary, Summary0), !,
-	new(Summary, string('%N: %s', Pred, Summary0)).
+	functor(Class, ClassName, _),
+	(   (   Class == autoload,
+		autoload_source(F, From)
+	    ->	true
+	    ;	Class = imported(From)
+	    ),
+	    file_name_on_path(From, Alias)
+	->  term_to_atom(Alias, Atom),
+	    new(Summary, string('%N: [%s from %s] %s', Pred, ClassName, Atom, Summary0))
+	;   new(Summary, string('%N: [%s] %s', Pred, ClassName, Summary0))
+	).
 identify_pred(built_in, F, Summary) :-
 	get(F, head, Head),
 	predicate_property(system:Head, foreign), !,
@@ -1517,16 +1554,10 @@ identify_pred(built_in, F, Summary) :-
 	sub_atom(Name, 0, _, _, $), !,
 	new(Summary, string('%N: SWI-Prolog private built-in', F)).
 identify_pred(autoload, F, Summary) :-	% Autoloaded predicates
-	get(F, name, Name),
-	get(F, arity, Arity),
-	'$find_library'(_Module, Name, Arity, _LoadModule, Library), !,
-	absolute_file_name(Library,
-			   [ file_type(prolog),
-			     access(read),
-			     file_errors(fail)
-			   ],
-			   Source),
-	new(Summary, string('%N: autoload from %s', F, Source)).
+	autoload_source(F, Source),
+	file_name_on_path(Source, Alias),
+	term_to_atom(Alias, Atom),
+	new(Summary, string('%N: autoload from %s', F, Atom)).
 identify_pred(local(Line), F, Summary) :-	% Local predicates
 	new(Summary, string('%N: locally defined at line %d', F, Line)).
 identify_pred(foreign(Line), F, Summary) :-	% Foreign predicates
@@ -1534,7 +1565,9 @@ identify_pred(foreign(Line), F, Summary) :-	% Foreign predicates
 identify_pred(constraint(Line), F, Summary) :-	% Local constraint
 	new(Summary, string('%N: constraint defined at line %d', F, Line)).
 identify_pred(imported(From), F, Summary) :-
-	new(Summary, string('%N: imported from %s', F, From)).
+	file_name_on_path(From, Alias),
+	term_to_atom(Alias, Atom),
+	new(Summary, string('%N: imported from %s', F, Atom)).
 identify_pred(recursion, _, 'Recursive reference') :- !.
 identify_pred(dynamic(_Line), F, Summary) :-
 	get(F, loaded_specifier, Spec),
@@ -1548,6 +1581,17 @@ identify_pred(global, _, 'Predicate defined in global module user') :- !.
 identify_pred(Class, _, ClassName) :-
 	term_to_atom(Class, ClassName).
 
+autoload_source(F, Source) :-
+	get(F, name, Name),
+	get(F, arity, Arity),
+	'$find_library'(_Module, Name, Arity, _LoadModule, Library), !,
+	absolute_file_name(Library,
+			   [ file_type(prolog),
+			     access(read),
+			     file_errors(fail)
+			   ],
+			   Source).
+
 :- pce_end_class(emacs_goal_fragment).
 
 
@@ -1780,6 +1824,8 @@ identify_fragment(head(exported), _, 'Exported (Public) predicate').
 identify_fragment(head(multifile), _, 'Multifile predicate').
 identify_fragment(head(constraint), _, 'Constraint').
 identify_fragment(prolog_data, _, 'Pass Prolog term unmodified').
+identify_fragment(keyword(except), _, 'Import all except given').
+identify_fragment(keyword(as), _, 'Import under a different name').
 identify_fragment(Class, _, Summary) :-
 	term_to_atom(Class, Summary).
 
