@@ -1,4 +1,4 @@
-/*  $Id: popupgesture.c,v 1.15 2003/09/04 14:02:13 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -62,7 +62,9 @@ updatePopupGesture(PopupGesture g, EventObj ev)
     } else
       p = g->popup;
   } else
-  { TRY( p = get(rec, NAME_popup, EAV) );
+  { if ( !(p = get(rec, NAME_popup, EAV)) ||
+	 !instanceOfObject(p, ClassPopup) )
+      fail;
   }
 
   assign(g, current, p);
