@@ -1,4 +1,4 @@
-/*  $Id: interface.c,v 1.37 2003/08/01 09:03:06 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -48,7 +48,7 @@
 		********************************/
 
 Any
-cToPceInteger(long int i)
+cToPceInteger(intptr_t i)
 { Int n = toInt(i);
 
   if ( valInt(n) != i )
@@ -67,7 +67,7 @@ cToPceReal(double f)
 
 
 Any
-cToPceString(Name assoc, const char *s, unsigned int len, int translate)
+cToPceStringA(Name assoc, const char *s, size_t len, int translate)
 { Any str;
   string ss;
   Any c;
@@ -90,6 +90,30 @@ cToPceString(Name assoc, const char *s, unsigned int len, int translate)
 }
 
 
+Any
+cToPceStringW(Name assoc, const wchar_t *s, size_t len, int translate)
+{ Any str;
+  string ss;
+  Any c;
+
+  str_set_n_wchar(&ss, len, (wchar_t*)s);
+  c = StringToScratchCharArray(&ss);
+
+  if ( translate )
+    str = pceNew(assoc, ClassString, 1, &c);
+  else
+  { Any av[2];
+    
+    av[0] = name_procent_s;
+    av[1] = c;
+    str = pceNew(assoc, ClassString, 2, av);
+  }
+  doneScratchCharArray(c);
+
+  return str;
+}
+
+
 Any
 cToPceName(const char *text)
 { if ( text )
@@ -104,7 +128,7 @@ cToPceName(const char *text)
 
 
 Any
-cToPceName_n(const char *text, unsigned int len)
+cToPceName_nA(const char *text, size_t len)
 { if ( text )
   { string s;
 
@@ -116,6 +140,12 @@ cToPceName_n(const char *text, unsigned int len)
 }
 
 
+Any
+cToPceName_nW(const wchar_t *text, size_t len)
+{ return WCToName(text, len);
+}
+
+
 Any
 cToPcePointer(void *ptr)
 { CPointer p = answerObjectv(ClassCPointer, 0, NULL);
@@ -151,7 +181,7 @@ pceObjectFromName(PceName name)
 
 
 Any
-cToPceReference(unsigned long val)
+cToPceReference(uintptr_t val)
 { Instance rval = longToPointer(val);
 
   if ( rval &&
@@ -164,7 +194,7 @@ cToPceReference(unsigned long val)
 
 
 int
-pceExistsReference(unsigned long ref)
+pceExistsReference(uintptr_t ref)
 { Any addr = longToPointer(ref);
 
   if ( !isProperObject(addr) || isFreedObj(addr) )
@@ -182,7 +212,7 @@ pcePPReference(PceObject ref)
 
     if ( rval[0] != '@' )
     { char tmp[256];
-      sprintf(tmp, "@%ld", valInt(ref));
+      sprintf(tmp, "@" INTPTR_FORMAT, valInt(ref));
       return save_string(tmp);
     } else
       return rval;
@@ -371,27 +401,40 @@ pceToC(Any obj, PceCValue *rval)
 }
 
 
-char *
-pceStringToC(Any val)
-{ if ( instanceOfObject(val, ClassString) )
-  { StringObj str = val;
+int
+pceIsString(Any val)
+{ return instanceOfObject(val, ClassString) ? TRUE : FALSE;
+}
 
-    return strName(str);
-  }  
+
+char *
+pceCharArrayToCA(Any val, size_t *len)
+{ if ( instanceOfObject(val, ClassCharArray) )
+  { CharArray ca = val;
+
+    if ( isstrA(&ca->data) )
+    { if ( len )
+	*len = ca->data.size;
+
+      return (char*)ca->data.s_textA;
+    }
+  }
 
   return NULL;
 }
 
 
-char *
-pceCharArrayToC(Any val, unsigned int *len)
+wchar_t *
+pceCharArrayToCW(Any val, size_t *len)
 { if ( instanceOfObject(val, ClassCharArray) )
   { CharArray ca = val;
 
-    if ( len )
-      *len = (unsigned int)str_datasize(&ca->data);
+    if ( isstrW(&ca->data) )
+    { if ( len )
+	*len = ca->data.size;
 
-    return strName(ca);
+      return ca->data.s_textW;
+    }
   }
 
   return NULL;
@@ -803,6 +846,25 @@ Cputchar(int chr)
 }
 
 
+int
+Cputstr(String s)
+{ if ( TheCallbackFunctions.Cputchar )
+  { int i;
+
+    for(i=0; i<s->size; i++)
+    { (*TheCallbackFunctions.Cputchar)(str_fetch(s, i));
+    }
+
+    return s->size;
+  } else if ( isstrA(s) )
+  { Cprintf("%s", s->s_textA);
+
+    return s->size;
+  } else
+    return 0;
+}
+
+
 void
 Cflush()
 { if ( TheCallbackFunctions.Cflush )
@@ -844,13 +906,13 @@ pceSetProfileHooks(pce_profile_hooks *hooks)
 #undef pceFree
 
 void *
-pceMalloc(int size)
+pceMalloc(size_t size)
 { return (*TheCallbackFunctions.malloc)(size);
 }
 
 
 void *
-pceRealloc(void *ptr, int size)
+pceRealloc(void *ptr, size_t size)
 { return (*TheCallbackFunctions.realloc)(ptr, size);
 }
 
