@@ -1,4 +1,4 @@
-/*  $Id: regex.c,v 1.15 2002/02/01 15:05:01 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -24,32 +24,38 @@
 
 #include <h/kernel.h>
 #include <h/text.h>		/* text_buffer */
-#include <gnu/gregex.h>
+#include <rgx/regex.h>
 
 forwards Int getRegisterStartRegex(Regex, Int);
 
-#define BYTEWIDTH 8
-#define FASTMAPSIZE (1<<BYTEWIDTH)
-
 NewClass(regex)
   CharArray		    pattern;	/* Pattern matched */
-  struct re_pattern_buffer *compiled;	/* Compiled regex */
-  struct re_registers	   *registers;	/* \0-\9 matches */
+  Name			    syntax;	/* basic, extended, advanced */
+  Bool			    ignore_case;/* @on --> case insensitive */
+  Int			    re_flags;	/* REG_* FLAGS */
+  regex_t		   *compiled;	/* compiled regex */
+  regmatch_t		   *registers;	/* \0-\9 matches */
 End;
 
 
 static status
-initialiseRegex(Regex re, CharArray pattern, Bool case_sensitive)
+initialiseRegex(Regex re, CharArray pattern,
+		Bool case_sensitive,
+		Name syntax)
 { if ( isDefault(pattern) )
     pattern = (CharArray)NAME_;
+  if ( isDefault(syntax) )
+    syntax = NAME_advanced;
   assign(re, pattern, pattern);
 
-  re->registers    = NULL;
-  re->compiled     = alloc(sizeof(struct re_pattern_buffer));
-  memset(re->compiled, 0, sizeof(struct re_pattern_buffer));
+  if ( case_sensitive == OFF )
+    assign(re, ignore_case, ON);
+  else					/* @default, @on */
+    assign(re, ignore_case, OFF);
+  assign(re, syntax, syntax);
 
-  if ( notDefault(case_sensitive) )
-    ignoreCaseRegex(re, case_sensitive == ON ? OFF : ON);
+  re->compiled  = NULL;
+  re->registers = NULL;
 
   succeed;
 }
@@ -58,37 +64,18 @@ initialiseRegex(Regex re, CharArray pattern, Bool case_sensitive)
 static void
 unlink_registers(Regex re)
 { if ( re->registers != NULL )
-  {
-    if ( re->registers->start )
-      pceFree(re->registers->start);	/* malloc'ed by gnu-regex.c */
-    if ( re->registers->end )
-      pceFree(re->registers->end);		/* same */
-    unalloc(sizeof(struct re_registers), re->registers);
+  { pceFree(re->registers);
     re->registers = NULL;
   }
 }
 
 
-static void
-unlink_compiled_buffer(Regex re)
-{ if ( re->compiled != NULL )
-  { if ( re->compiled->buffer != NULL )
-    { pceFree(re->compiled->buffer);		/* malloc'ed by gnu-regex.c */
-      re->compiled->buffer = NULL;
-    }
-    if ( re->compiled->fastmap != NULL )
-    { pceFree(re->compiled->fastmap);
-      re->compiled->fastmap = NULL;
-    }
-  }
-}
-
-
 static void
 unlink_compiled(Regex re)
 { if ( re->compiled != NULL )
-  { unlink_compiled_buffer(re);
-    unalloc(sizeof(struct re_pattern_buffer), re->compiled);
+  { regfree(re->compiled);
+    pceFree(re->compiled);
+
     re->compiled = NULL;
   }
 }
@@ -114,8 +101,7 @@ loadRegex(Regex re, IOSTREAM *fd, ClassDef def)
 { TRY(loadSlotsObject(re, fd, def));
 
   re->registers	  	  = NULL;
-  re->compiled 		  = alloc(sizeof(struct re_pattern_buffer));
-  memset(re->compiled, 0, sizeof(struct re_pattern_buffer));
+  re->compiled 		  = NULL;
 
   succeed;
 }
@@ -126,8 +112,7 @@ cloneRegex(Regex re, Regex clone)
 { clonePceSlots(re, clone);
 
   re->registers		  = NULL;
-  re->compiled 		  = alloc(sizeof(struct re_pattern_buffer));
-  memset(re->compiled, 0, sizeof(struct re_pattern_buffer));
+  re->compiled 		  = NULL;
 
   succeed;
 }
@@ -144,60 +129,32 @@ patternRegex(Regex re, StringObj pattern)
 { assign(re, pattern, pattern);
 
   unlink_registers(re);
-  unlink_compiled_buffer(re);
+  unlink_compiled(re);
 
   succeed;
 }
 
 
-static char upcase[0400] = 
-{ 000, 001, 002, 003, 004, 005, 006, 007,
-  010, 011, 012, 013, 014, 015, 016, 017,
-  020, 021, 022, 023, 024, 025, 026, 027,
-  030, 031, 032, 033, 034, 035, 036, 037,
-  040, 041, 042, 043, 044, 045, 046, 047,
-  050, 051, 052, 053, 054, 055, 056, 057,
-  060, 061, 062, 063, 064, 065, 066, 067,
-  070, 071, 072, 073, 074, 075, 076, 077,
-  0100, 0101, 0102, 0103, 0104, 0105, 0106, 0107,
-  0110, 0111, 0112, 0113, 0114, 0115, 0116, 0117,
-  0120, 0121, 0122, 0123, 0124, 0125, 0126, 0127,
-  0130, 0131, 0132, 0133, 0134, 0135, 0136, 0137,
-  0140, 0101, 0102, 0103, 0104, 0105, 0106, 0107,
-  0110, 0111, 0112, 0113, 0114, 0115, 0116, 0117,
-  0120, 0121, 0122, 0123, 0124, 0125, 0126, 0127,
-  0130, 0131, 0132, 0173, 0174, 0175, 0176, 0177,
-  0200, 0201, 0202, 0203, 0204, 0205, 0206, 0207,
-  0210, 0211, 0212, 0213, 0214, 0215, 0216, 0217,
-  0220, 0221, 0222, 0223, 0224, 0225, 0226, 0227,
-  0230, 0231, 0232, 0233, 0234, 0235, 0236, 0237,
-  0240, 0241, 0242, 0243, 0244, 0245, 0246, 0247,
-  0250, 0251, 0252, 0253, 0254, 0255, 0256, 0257,
-  0260, 0261, 0262, 0263, 0264, 0265, 0266, 0267,
-  0270, 0271, 0272, 0273, 0274, 0275, 0276, 0277,
-  0300, 0301, 0302, 0303, 0304, 0305, 0306, 0307,
-  0310, 0311, 0312, 0313, 0314, 0315, 0316, 0317,
-  0320, 0321, 0322, 0323, 0324, 0325, 0326, 0327,
-  0330, 0331, 0332, 0333, 0334, 0335, 0336, 0337,
-  0340, 0341, 0342, 0343, 0344, 0345, 0346, 0347,
-  0350, 0351, 0352, 0353, 0354, 0355, 0356, 0357,
-  0360, 0361, 0362, 0363, 0364, 0365, 0366, 0367,
-  0370, 0371, 0372, 0373, 0374, 0375, 0376, 0377
-};
+status
+ignoreCaseRegex(Regex re, Bool val)
+{ if ( re->ignore_case != val )
+  { assign(re, ignore_case, val);
 
+    unlink_registers(re);
+    unlink_compiled(re);		/* force recompilation */
+  }
 
-static Bool
-getIgnoreCaseRegex(Regex re)
-{ answer(re->compiled->translate == NULL ? OFF : ON);
+  succeed;
 }
 
 
-status
-ignoreCaseRegex(Regex re, Bool val)
-{ if ( getIgnoreCaseRegex(re) != val )
-  { re->compiled->translate = (val == OFF ? NULL : upcase);
+static status
+syntaxRegex(Regex re, Name syntax)
+{ if ( re->syntax != syntax )
+  { assign(re, syntax, syntax);
+
     unlink_registers(re);
-    unlink_compiled_buffer(re);		/* force recompilation */
+    unlink_compiled(re);		/* force recompilation */
   }
 
   succeed;
@@ -206,133 +163,263 @@ ignoreCaseRegex(Regex re, Bool val)
 
 status
 compileRegex(Regex re, Bool optimize)
-{ if ( re->compiled->buffer == NULL )
-  { char *error;
+{ succeed;				/* backward compatibility */
+}
 
-    re->compiled->allocated = 0;
-    re->compiled->fastmap   = (optimize == ON ? pceMalloc(FASTMAPSIZE) : NULL);
 
-    re_set_syntax(RE_SYNTAX_EMACS);
-    if ( (error = (char *) re_compile_pattern(re->pattern->data.s_text,
-					      re->pattern->data.size,
-					      re->compiled)) != NULL )
-      return errorPce(re, NAME_syntaxError, CtoName(error));
+static status
+error_regex(Regex re, int rc)
+{ if ( rc != REG_NOMATCH )
+  { char buf[1024];
 
-    re->registers = alloc(sizeof(struct re_registers));
-    re->registers->start = re->registers->end = NULL;
-    re->compiled->regs_allocated = REGS_UNALLOCATED;
+    regerror(rc, re->compiled, buf, sizeof(buf));
+
+    return errorPce(re, NAME_syntaxError, CtoName(buf));
+  }
+
+  fail;					/* normal match failure */
+}
+
+
+#define RE_MATCH	0x0001
+#define RE_SEARCH	0x0002
+
+static status
+ensure_compiled_regex(Regex re, int flags)
+{ int myflags = REG_NLANCH;		/* make ^$ match around \n */
+
+  if ( re->ignore_case == ON )
+    myflags |= REG_ICASE;
+  if ( (flags & RE_MATCH) )
+    myflags |= REG_BOSONLY;
+
+  if ( re->syntax == NAME_basic )
+    myflags |= REG_BASIC;
+  else if ( re->syntax == NAME_extended )
+    myflags |= REG_EXTENDED;
+  else
+    myflags |= REG_ADVANCED;
+
+  if ( !re->compiled ||
+       isNil(re->re_flags) ||
+       valInt(re->re_flags) != myflags )
+  { int rc;
+    wchar_t *ws;
+    size_t len;
+
+    unlink_compiled(re);
+    unlink_registers(re);
+
+    ws = charArrayToWC(re->pattern, &len);
+    re->compiled = pceMalloc(sizeof(regex_t));
+    rc = re_compileW(re->compiled, ws, len, myflags);
+
+    if ( rc == REG_OKAY )
+    { re->registers = pceMalloc(sizeof(regmatch_t)*(re->compiled->re_nsub+1));
+      assign(re, re_flags, toInt(myflags));
+    } else
+    { error_regex(re, rc);
+      pceFree(re->compiled);
+      re->compiled = NULL;
+
+      fail;
+    }
   }
 
   succeed;
 }
 
 
+
+
+#define IDXOFF		0x1000
+#define IDX2PTR(i)	(charW*)((i)*sizeof(charW)+IDXOFF)
+#define PTR2IDX(p)	((long)(p)-IDXOFF)/sizeof(charW)
+
+
+static int
+re_fetch_string(const charW *at, void *closure)
+{ String str = closure;
+  long idx = PTR2IDX(at);
+
+  return str_fetch(str, idx);
+}
+
+
+static int
+re_fetch_textbuffer(const charW *at, void *closure)
+{ TextBuffer tb = closure;
+  long idx = PTR2IDX(at);
+
+  return fetch_textbuffer(tb, idx);
+}
+
+
+static int
+re_fetch_fragment(const charW *at, void *closure)
+{ Fragment f = closure;
+  long idx = PTR2IDX(at);
+
+  return fetch_textbuffer(f->textbuffer, idx+f->start);
+}
+
+
 status
-search_regex(Regex re, char *str1, int size1, char *str2, int size2,
-	     int start, int end)
-{ TRY(compileRegex(re, size1+size2 > 100 ? ON : OFF));
+search_string_regex(Regex re, String s)
+{ int rc;
 
-  if ( end < start )
-  { for(;;)
-    { switch( re_search_2(re->compiled,
-			  str1, size1, str2, size2,
-			  start, end-start, re->registers, start) )
-      { case -1:
-	  fail;
-	case -2:
-	  return errorPce(re, NAME_internalError);
-	default:
-	  if ( re->registers->end[0] > start )
-	  { DEBUG(NAME_regex, Cprintf("Match to %d\n", re->registers->end[0]));
+  if ( !ensure_compiled_regex(re, RE_SEARCH) )
+    fail;
+  
+  rc = re_execW(re->compiled, IDX2PTR(0), s->size,
+		re_fetch_string, s,
+		NULL,
+		re->compiled->re_nsub+1, re->registers, 0);
 
-	    if ( --start < 0 )
-	      fail;
+  if ( rc == 0 )
+    succeed;
+  else
+    return error_regex(re, rc);
+}
+
+
+
+static status
+search_regex(Regex re, Any obj, Int start, Int end, int flags)
+{ int from = isDefault(start) ? 0 : valInt(start);
+  int len, to, eflags = 0;
+  int (*fetch)(const charW*, void*);
+  void *closure;
+
+  if ( instanceOfObject(obj, ClassCharArray) )
+  { CharArray ca = obj;			/* TBD: 16-bit? */
+    String s = &ca->data;
+
+    len = s->size;
+    fetch = re_fetch_string;
+    closure = s;
+  } else if ( instanceOfObject(obj, ClassTextBuffer) )
+  { TextBuffer tb = obj;
+    
+    len = tb->size;
+    fetch = re_fetch_textbuffer;
+    closure = tb;
+
+  } else if ( instanceOfObject(obj, ClassFragment) )
+  { Fragment frag = obj;
+
+    len = frag->length;
+    fetch = re_fetch_fragment;
+    closure = frag;
+  } else
+    fail;
+   
+  if ( isDefault(end) )
+  { to = len;
+  } else
+  { to = valInt(end);
+    if ( to < 0   ) to = 0;
+    if ( to > len ) to = len;
+  }
+
+  if ( isDefault(start) )
+  { from = 0;
+  } else
+  { from = valInt(start);
+    if ( from < 0   ) from = 0;
+    if ( from > len ) from = len;
+  }
+
+  if ( from <= to )			/* forwards */
+  { int rc;
+
+					  /* partial match, check ends */
+    if ( from > 0 && (*fetch)(IDX2PTR(from-1), closure) != '\n' )
+      eflags |= REG_NOTBOL;
+    if ( to < len && (*fetch)(IDX2PTR(to), closure) != '\n' )
+      eflags |= REG_NOTEOL;
+
+    if ( !ensure_compiled_regex(re, flags) ) /* RE_SEARCH/RE_MATCH */
+      fail;
+
+    rc = re_execW(re->compiled, IDX2PTR(from), to-from,
+		  fetch, closure,
+		  NULL,
+		  re->compiled->re_nsub+1, re->registers, eflags);
+    if ( rc == 0 )
+    { if ( from != 0 )
+      { size_t n;
+
+	for(n=0; n <= re->compiled->re_nsub; n++)
+	{ re->registers[n].rm_so += from;
+	  re->registers[n].rm_eo += from;
+	}
+      }
+      succeed;
+    } else
+      return error_regex(re, rc);
+  } else				/* backward search */
+  { int rc;
+    int here;
+    int match = -1;
+
+    if ( !ensure_compiled_regex(re, RE_MATCH) )
+      fail;
+
+    if ( from < len && (*fetch)(IDX2PTR(from), closure) != '\n' )
+      eflags |= REG_NOTEOL;
+
+    for(here = from; here >= to; here--)
+    { eflags &= ~REG_NOTBOL;
+
+      if ( here > 0 && (*fetch)(IDX2PTR(here-1), closure) != '\n' )
+	eflags |= REG_NOTBOL;
+
+      rc = re_execW(re->compiled, IDX2PTR(here), from-here,
+		    fetch, closure,
+		    NULL,
+		    re->compiled->re_nsub+1, re->registers, eflags);
+      switch(rc)
+      { case REG_NOMATCH:
+	{ size_t n;
+
+	  if ( match == -1 )
 	    continue;
+	  rc = re_execW(re->compiled, IDX2PTR(match), from-match,
+			fetch, closure,
+			NULL,
+			re->compiled->re_nsub+1, re->registers, 0);
+	  assert(rc == REG_OKAY);
+	adjust:
+	  if ( flags == RE_MATCH )
+	  { if ( match+re->registers[0].rm_eo != from )
+	      fail;
+	  }
+
+	  for(n=0; n <= re->compiled->re_nsub; n++)
+	  { re->registers[n].rm_so += match;
+	    re->registers[n].rm_eo += match;
 	  }
 
 	  succeed;
+	}
+	case REG_OKAY:
+	  match = here;
+	  if ( here == to )
+	    goto adjust;
+	  continue;
+	default:
+	  return error_regex(re, rc);
       }
     }
-  } else
-  { switch( re_search_2(re->compiled,
-			str1, size1, str2, size2,
-			start, end-start, re->registers, end) )
-    { case -1:
-	fail;
-      case -2:
-	return errorPce(re, NAME_internalError);
-      default:
-	succeed;
-    }
+    fail;			/* no match */
   }
 }
 
 
 status
 searchRegex(Regex re, Any obj, Int start, Int end)
-{ int from = isDefault(start) ? 0 : valInt(start);
-
-  if ( instanceOfObject(obj, ClassCharArray) )
-  { CharArray ca = obj;			/* TBD: 16-bit? */
-    char *s = (char *)ca->data.s_text;
-    int ls = ca->data.size;
-
-    return search_regex(re, s, ls, NULL, 0,
-			from, isDefault(end) ? ls : valInt(end));
-  } else if ( instanceOfObject(obj, ClassTextBuffer) )
-  { TextBuffer tb = obj;
-
-    if ( tb->buffer.b16 )
-      return search_regex(re,
-			  (char *)&tb->tb_buffer16[0],
-			  tb->gap_start*2,
-			  (char *)&tb->tb_buffer16[tb->gap_end+1],
-			  (tb->size - tb->gap_start)*2,
-			  from*2,
-			  (isDefault(end) ? tb->size : valInt(end))*2);
-    else
-      return search_regex(re,
-			  (char *)&tb->tb_buffer8[0],
-			  tb->gap_start,
-			  (char *)&tb->tb_buffer8[tb->gap_end+1],
-			  tb->size - tb->gap_start,
-			  from,
-			  isDefault(end) ? tb->size : valInt(end));
-  } else if ( instanceOfObject(obj, ClassFragment) )
-  { Fragment frag = obj;
-    TextBuffer tb = frag->textbuffer;
-    status rval;
-
-    if ( tb->buffer.b16 )
-      rval = search_regex(re,
-			  (char *)tb->tb_buffer16, tb->gap_start*2,
-			  (char *)&tb->tb_buffer16[tb->gap_end+1],
-			  (tb->size - tb->gap_start)*2,
-			  (from+frag->start)*2,
-			  (isDefault(end) ? frag->start + frag->length
-			  		  : valInt(end)+frag->start)*2);
-    else
-      rval = search_regex(re,
-			  (char *)tb->tb_buffer8, tb->gap_start,
-			  (char *)&tb->tb_buffer8[tb->gap_end+1],
-			  tb->size - tb->gap_start,
-			  from+frag->start,
-			  isDefault(end) ? frag->start + frag->length
-			  		 : valInt(end)+frag->start);
-
-    if ( rval )
-    { int n;
-
-      for(n=0; re->registers->start[n] >= 0; n++)
-      { re->registers->start[n] -= frag->start;
-	re->registers->end[n] -= frag->start;
-      }
-    }
-
-    return rval;
-  }
-   
-  fail;
+{ return search_regex(re, obj, start, end, RE_SEARCH);
 }
 
 
@@ -344,111 +431,11 @@ getSearchRegex(Regex re, Any obj, Int start, Int end)
 }
 
 
-static Int
-match_regex(Regex re, char *str1, int size1, char *str2, int size2, int start, int end)
-{ int n;
-  TRY(compileRegex(re, size1+size2 > 100 ? ON : OFF));
-
-  if ( end >= start )			/* forward */
-  { switch( (n = re_match_2(re->compiled,
-			    (unsigned char *)str1, size1,
-			    (unsigned char *)str2, size2,
-			    start, re->registers, end)) )
-    { case -1:
-	fail;
-      case -2:
-	{ errorPce(re, NAME_internalError);
-	  fail;
-	}
-      default:
-	return toInt(n);
-    }
-  } else				/* backward */
-  { int s = start;
-    int n;
-
-    for(s = start; s >= end; s--)
-    { DEBUG(NAME_regex, Cprintf("Match in %d .. %d\n", s, start));
-      switch( (n = re_match_2(re->compiled,
-			      (unsigned char *)str1, size1,
-			      (unsigned char *)str2, size2,
-			      s, re->registers, start)) )
-      { case -1:
-	  continue;
-	case -2:
-	  { errorPce(re, NAME_internalError);
-	    fail;
-	  }
-	default:
-	  if ( s + n == start )
-	    return toInt(n);
-      }
-    }
-    fail;
-  }
-}
-
-
 Int
 getMatchRegex(Regex re, Any obj, Int start, Int end)
-{ int from = isDefault(start) ? 0 : valInt(start);
-
-  if ( instanceOfObject(obj, ClassCharArray) )
-  { CharArray ca = obj;
-    char *s = (char *)ca->data.s_text;
-    int ls = ca->data.size;
-
-    return match_regex(re, s, ls, NULL, 0,
-		       from, isDefault(end) ? ls : valInt(end));
-  } else if ( instanceOfObject(obj, ClassTextBuffer) )
-  { TextBuffer tb = obj;
-
-    if ( tb->buffer.b16 )
-      return match_regex(re,
-			 (char *)tb->tb_buffer16, tb->gap_start*2,
-			 (char *)&tb->tb_buffer16[tb->gap_end+1],
-			 (tb->size - tb->gap_start)*2,
-			 from*2,
-			 (isDefault(end) ? tb->size : valInt(end))*2);
-    else
-      return match_regex(re,
-			 (char *)tb->tb_buffer8, tb->gap_start,
-			 (char *)&tb->tb_buffer8[tb->gap_end+1],
-			 tb->size - tb->gap_start,
-			 from, isDefault(end) ? tb->size : valInt(end));
-  } else if ( instanceOfObject(obj, ClassFragment) )
-  { Fragment frag = obj;
-    TextBuffer tb = frag->textbuffer;
-    Int rval;
-
-    if ( tb->buffer.b16 )
-      rval = match_regex(re,
-			 (char *)tb->tb_buffer16, tb->gap_start*2,
-			 (char *)&tb->tb_buffer16[tb->gap_end+1],
-			 (tb->size - tb->gap_start)*2,
-			 (from+frag->start)*2,
-			 (isDefault(end) ? frag->start + frag->length
-			 		 : valInt(end)+frag->start)*2);
-    else
-      rval = match_regex(re,
-			 (char *)tb->tb_buffer8, tb->gap_start,
-			 (char *)&tb->tb_buffer8[tb->gap_end+1],
-			 tb->size - tb->gap_start,
-			 from+frag->start,
-			 isDefault(end) ? frag->start + frag->length
-			 		: valInt(end)+frag->start);
-    
-    if ( rval )
-    { int n;
-
-      for(n=0; re->registers->start[n] >= 0; n++)
-      { re->registers->start[n] -= frag->start;
-	re->registers->end[n] -= frag->start;
-      }
-    }
-
-    answer(rval);
-  }
+{ if ( search_regex(re, obj, start, end, RE_MATCH) )
+    answer(toInt(re->registers[0].rm_eo -
+		 re->registers[0].rm_so));
    
   fail;
 }
@@ -461,8 +448,8 @@ matchRegex(Regex re, Any obj, Int start, Int end)
 
 
 #define validRegister(re, n) ((n) >= 0 && \
-			      re->registers && \
-			      re->registers->start[(n)] >= 0)
+			      re->compiled && \
+			      (n) <= (int)re->compiled->re_nsub)
 
 
 static Int
@@ -472,7 +459,7 @@ getRegisterStartRegex(Regex re, Int which)
   if ( !validRegister(re, n) )
     fail;
 
-  answer(toInt(re->registers->start[n]));
+  answer(toInt(re->registers[n].rm_so));
 }
 
 
@@ -483,7 +470,7 @@ getRegisterEndRegex(Regex re, Int which)
   if ( !validRegister(re, n) )
     fail;
 
-  answer(toInt(re->registers->end[n]));
+  answer(toInt(re->registers[n].rm_eo));
 }
 
 
@@ -494,7 +481,7 @@ getRegisterSizeRegex(Regex re, Int which)
   if ( !validRegister(re, n) )
     fail;
 
-  answer(toInt(re->registers->end[n] - re->registers->start[n]));
+  answer(toInt(re->registers[n].rm_eo - re->registers[n].rm_so));
 }
 
 
@@ -507,8 +494,8 @@ getRegisterValueRegex(Regex re, Any obj, Int which, Type type)
   if ( !validRegister(re, n) )
     fail;
 
-  argv[0] = toInt(re->registers->start[n]);
-  argv[1] = toInt(re->registers->end[n]);
+  argv[0] = toInt(re->registers[n].rm_so);
+  argv[1] = toInt(re->registers[n].rm_eo);
 
   rval = getv(obj, NAME_sub, 2, argv);
   if ( rval && notDefault(type) )
@@ -527,8 +514,8 @@ registerValueRegex(Regex re, Any obj, CharArray value, Int which)
   if ( !validRegister(re, n) )
     fail;
 
-  start = re->registers->start[n];
-  len   = re->registers->end[n] - start;
+  start = re->registers[n].rm_so;
+  len   = re->registers[n].rm_eo - start;
   shift = valInt(getSizeCharArray(value)) - len;
 
   argv[0] = toInt(start);
@@ -537,11 +524,13 @@ registerValueRegex(Regex re, Any obj, CharArray value, Int which)
   if ( sendv(obj, NAME_delete, 2, argv) &&
        (argv[1] = value) &&
        sendv(obj, NAME_insert, 2, argv) )
-  { for(n=0; re->registers->start[n] >= 0; n++)
-    { if ( re->registers->start[n] > start )
-	re->registers->start[n] += shift;
-      if ( re->registers->end[n] >= start )
-	re->registers->end[n] += shift;
+  { size_t i;
+
+    for(i=0; i <= re->compiled->re_nsub; i++)
+    { if ( re->registers[i].rm_so > start )
+	re->registers[i].rm_so += shift;
+      if ( re->registers[i].rm_eo >= start )
+	re->registers[i].rm_eo += shift;
     }
     
     succeed;
@@ -554,17 +543,19 @@ registerValueRegex(Regex re, Any obj, CharArray value, Int which)
 static status
 replaceRegex(Regex re, Any obj, CharArray value)
 { String s = &value->data;
-  LocalString(buf, s, FORMATSIZE);
+  LocalString(buf, s->iswide, FORMATSIZE);
   int o, i, size = s->size;
   CharArray repl;
   status rval;
 
   for(i=o=0; i<size; i++)
-  { wchar c = str_fetch(s, i);
+  { wint_t c = str_fetch(s, i);
+    wint_t c2;
 
-    if ( c == '\\' && isdigit(str_fetch(s, i+1)) )
+					/* only POSIX '0'..'9' */
+    if ( c == '\\' && (c2=str_fetch(s, i+1)) >= '0' && c2 <= '9' )
     { CharArray ca;
-      Int reg = toInt(str_fetch(s, i+1) - '0');
+      Int reg = toInt(c2 - '0');
 
       if ( (ca = getRegisterValueRegex(re, obj, reg, DEFAULT)) )
       { str_ncpy(buf, o, &ca->data, 0, ca->data.size);
@@ -605,12 +596,12 @@ forAllRegex(Regex re, Any obj, Code code, Int from, Int to)
     from = ZERO;
 
   while( searchRegex(re, obj, from, to) )
-  { int oe = re->registers->end[0];
+  { int oe = re->registers[0].rm_eo;
     int ne;
 
     TRY(forwardCode(code, re, obj, EAV));
 
-    ne = re->registers->end[0];
+    ne = re->registers[0].rm_eo;
     if ( ne == valInt(from) && oe == valInt(from) )
       from = toInt(ne+1);
     else
@@ -631,14 +622,14 @@ static StringObj
 getQuoteRegex(Regex re, CharArray ca)
 { String s = &ca->data;
   int size = s->size;
-  LocalString(buf, s, LINESIZE);
+  LocalString(buf, s->iswide, LINESIZE);
   int i, o=0;
 
   if ( str_fetch(s, 0) == '^' )
     str_store(buf, o++, '\\');
 
   for( i=0; i < size; i++)
-  { wchar c = str_fetch(s, i);
+  { wint_t c = str_fetch(s, i);
 
     switch(c)
     { case '?':
@@ -648,6 +639,8 @@ getQuoteRegex(Regex re, CharArray ca)
       case '*':
       case '+':
       case '.':
+      case '{':
+      case '}':
 	str_store(buf, o++, '\\');
 	break;
       case '$':
@@ -726,7 +719,9 @@ filePatternRegex(Regex re, CharArray file_pattern)
 /* Type declarations */
 
 static char *T_initialise[] =
-	{ "pattern=[char_array]", "case_sensitive=[bool]" };
+	{ "pattern=[char_array]", "case_sensitive=[bool]",
+	  "syntax=[{basic,extended,advanced}]"
+	};
 static char *T_forAll[] =
         { "in=char_array|text_buffer|fragment",
 	  "action=code", "from=[int]", "to=[int]" };
@@ -742,18 +737,24 @@ static char *T_sregisterValue[] =
 /* Instance Variables */
 
 static vardecl var_regex[] =
-{ IV(NAME_pattern, "char_array", IV_GET,
+{ SV(NAME_pattern, "char_array", IV_GET|IV_STORE, patternRegex,
      NAME_pattern, "Pattern to search for"),
-  IV(NAME_compiled, "alien:struct re_pattern_buffer *", IV_NONE,
-     NAME_compilation, "Buffer holding compiled pattern"),
-  IV(NAME_registers, "alien:struct re_registers *", IV_NONE,
-     NAME_registers, "Registers for (sub-)matches")
+  SV(NAME_syntax, "{basic,extended,advanced}", IV_GET|IV_STORE, syntaxRegex,
+     NAME_pattern, "Regex syntax used"),
+  SV(NAME_ignoreCase, "bool", IV_GET|IV_STORE, ignoreCaseRegex,
+     NAME_pattern, "Ignore case while matching?"),
+  IV(NAME_flags, "int*", IV_NONE,
+     NAME_internal, "REG_* flags"),
+  IV(NAME_compiled, "alien:regex_t*", IV_NONE,
+     NAME_internal, "Buffer holding compiled pattern"),
+  IV(NAME_registers, "alien:regmatch_t*", IV_NONE,
+     NAME_internal, "Registers for (sub-)matches")
 };
 
 /* Send Methods */
 
 static senddecl send_regex[] =
-{ SM(NAME_initialise, 2, T_initialise, initialiseRegex,
+{ SM(NAME_initialise, 3, T_initialise, initialiseRegex,
      DEFAULT, "Create regex from pattern"),
   SM(NAME_unlink, 0, NULL, unlinkRegex,
      DEFAULT, "Deallocate private storage"),
@@ -761,10 +762,6 @@ static senddecl send_regex[] =
      NAME_compilation, "Compile expression (optimized)"),
   SM(NAME_forAll, 4, T_forAll, forAllRegex,
      NAME_iterate, "Run code on each match"),
-  SM(NAME_ignoreCase, 1, "ignore=bool", ignoreCaseRegex,
-     NAME_pattern, "@on: case is ignored during match"),
-  SM(NAME_pattern, 1, "pattern=char_array", patternRegex,
-     NAME_pattern, "Set pattern searched for"),
   SM(NAME_filePattern, 1, "file_pattern=char_array", filePatternRegex,
      NAME_pattern, "Translate file-pattern to regex and ->pattern"),
   SM(NAME_registerValue, 3, T_sregisterValue, registerValueRegex,
@@ -782,8 +779,6 @@ static senddecl send_regex[] =
 static getdecl get_regex[] =
 { GM(NAME_convert, 1, "regex", "char_array", getConvertRegex,
      NAME_conversion, "Converts char_array to regex(pattern)"),
-  GM(NAME_ignoreCase, 0, "ignore=bool", NULL, getIgnoreCaseRegex,
-     NAME_pattern, "@on if case is ignored during match"),
   GM(NAME_quote, 1, "quoted=string", "pattern=char_array", getQuoteRegex,
      NAME_pattern, "Quoted version of argument"),
   GM(NAME_registerEnd, 1, "index=int", "register=[0..9]", getRegisterEndRegex,
