@@ -1,4 +1,4 @@
-/*  $Id: xdisplay.c,v 1.48 2006/04/07 11:24:23 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -635,7 +635,7 @@ collect_selection_display(Widget w, XtPointer xtp,
       charA *bufA, *outA;
       const charA *in = value;
       const charA *end = &in[*len];
-      wint_t chr;
+      int chr;
 
       if ( l > 8*1024*2024 )		/* sanity check */
       { selection_error = CtoName("Selection too long");
