@@ -33,7 +33,8 @@ This unit contains small tests for the garbage collector.
 test_gc :-
 	run_tests([ gc_leak,
 		    gc_reset,
-		    gc_crash
+		    gc_crash,
+		    agc
 		  ]).
 
 :- module_transparent
@@ -73,8 +74,14 @@ nospace(Goal) :-
 
 :- begin_tests(gc_leak, [sto(rational_trees)]).
 
-det_freeze :- 
-	freeze(X, X==1), X=1.
+det_freeze_loop(N, T) :-
+	(   succ(N2, N)
+	->  freeze(X, true),
+	    X = a,
+	    det_freeze_loop(N2, T)
+	;   garbage_collect,
+	    statistics(trailused, T)
+	).
 
 early_reset :-
 	early_reset(_).
@@ -82,8 +89,12 @@ early_reset :-
 early_reset(X) :- space(T,G), length(X, 10), must_space(T,G), !.
 early_reset(_) :- fail.
 
-test(attvar) :-
-	nospace(det_freeze).
+test(det_freeze_no_space) :-
+	garbage_collect,
+	statistics(trailused, T0),
+	det_freeze_loop(1000, T1),
+	T is T1 - T0,
+	T < 100.			% A small constant use is ok
 test(early_reset) :-
 	early_reset.
 
@@ -118,5 +129,51 @@ t("hello world").
 
 test(b_string) :-
 	t1.
+test(wakeup_two) :-
+	freeze(V1, true),
+	freeze(V2, (garbage_collect,V2==y)),
+	(   x(V1,V2) = x(a,b)
+	;   x(V1,V2) = x(x,y)
+	).
 
 :- end_tests(gc_crash).
+
+
+
+:- begin_tests(agc).
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Verify marking atoms from temporary clauses  for meta-calling. Using the
+enhanced garbage collector access through  the   variables  is no longer
+guaranteed.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+:- dynamic
+	v/1.
+
+test :-
+        atom_concat(abcd, efgh, A),
+        Goal = ( garbage_collect,
+                 garbage_collect_atoms,
+                 make_atoms,
+		 assert_atom(A)
+               ),
+	test_c(Goal).
+
+test_c(Goal) :-
+        Goal.
+
+assert_atom(A) :-
+	assert(v(A)).
+
+make_atoms :-
+        forall(between(1, 10000, X),
+               atom_concat(foo, X, _)).
+
+test(usercall, A == Ok) :-
+	retractall(v(_)),
+	test,
+	retract(v(A)),
+	atom_concat(abcd, efgh, Ok).
+		
+:- end_tests(agc).
