@@ -54,6 +54,55 @@ vName(Word adr)
 #endif
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+(*) Although this is an assignment from var   to value, we use a trailed
+assignment  to  exploit  mergeTrailedAssignments()   in  GC,  discarding
+multiple  assignments  in  the  same  segment,  needed  to  ensure  that
+deterministic wakeup does not leak  space.   The  test  program is this,
+which must run in constant space.
+
+	loop :- freeze(X, true), X = a, loop.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+static int
+registerWakeup(Word name, Word value ARG_LD)
+{ Word wake;
+  Word tail = valTermRef(LD->attvar.tail);
+
+  wake = allocGlobalNoShift(4);		/* may NOT shift the stacks!!! */
+  if ( !wake )
+  { outOfStack(&LD->stacks.global, STACK_OVERFLOW_THROW);
+    fail;
+  }
+  wake[0] = FUNCTOR_wakeup3;
+  wake[1] = needsRef(*name) ? makeRef(name) : *name;
+  wake[2] = needsRef(*value) ? makeRef(value) : *value;
+  wake[3] = ATOM_nil;
+
+  if ( *tail )
+  { Word t;				/* Non-empty list */
+
+    deRef2(tail, t);
+    TrailAssignment(t);
+    *t = consPtr(wake, TAG_COMPOUND|STG_GLOBAL);
+    TrailAssignment(tail);		/* on local stack! */
+    *tail = makeRef(wake+3);
+    DEBUG(1, Sdprintf("appended to wakeup\n"));
+  } else				/* empty list */
+  { Word head = valTermRef(LD->attvar.head);
+    
+    assert(isVar(*head));
+    TrailAssignment(head);		/* See (*) */
+    *head = consPtr(wake, TAG_COMPOUND|STG_GLOBAL);
+    TrailAssignment(tail);
+    *tail = makeRef(wake+3);
+    DEBUG(1, Sdprintf("new wakeup\n"));
+  }
+
+  succeed;
+}
+
+
 		 /*******************************
 		 *	     ASSIGNMENT		*
 		 *******************************/
@@ -74,9 +123,7 @@ function. If you change this you must also adjust unifiable/3.
 
 int
 assignAttVar(Word av, Word value ARG_LD)
-{ Word wake, a;				/*  */
-  Word tail = valTermRef(LD->attvar.tail);
-
+{ Word a;
   assert(isAttVar(*av));
   assert(!isRef(*value));
 
@@ -92,32 +139,8 @@ assignAttVar(Word av, Word value ARG_LD)
   }
 
   a = valPAttVar(*av);
-  wake    = allocGlobalNoShift(4);	/* may NOT shift the stacks!!! */
-  if ( !wake )
-    return outOfStack(&LD->stacks.global, STACK_OVERFLOW_THROW);
-  wake[0] = FUNCTOR_wakeup3;
-  wake[1] = needsRef(*a) ? makeRef(a) : *a;
-  wake[2] = needsRef(*value) ? makeRef(value) : *value;
-  wake[3] = ATOM_nil;
-
-  if ( *tail )
-  { Word t;				/* Non-empty list */
-
-    deRef2(tail, t);
-    TrailAssignment(t);
-    *t = consPtr(wake, TAG_COMPOUND|STG_GLOBAL);
-    DEBUG(1, Sdprintf("appended to wakeup\n"));
-  } else				/* empty list */
-  { Word head = valTermRef(LD->attvar.head);
-    
-    assert(isVar(*head));
-    *head = consPtr(wake, TAG_COMPOUND|STG_GLOBAL);
-    Trail(head);
-    DEBUG(1, Sdprintf("new wakeup\n"));
-  }
-
-  TrailAssignment(tail);		/* on local stack! */
-  *tail = makeRef(wake+3);
+  if ( !registerWakeup(a, value PASS_LD) )
+    fail;
 
   TrailAssignment(av);
   if ( isAttVar(*value) )
@@ -153,6 +176,34 @@ make_new_attvar(Word p ARG_LD)
 }
 
 
+static int
+put_new_attvar(Word p, atom_t name, Word value ARG_LD)
+{ Word gp, at;
+
+  if ( onStackArea(local, p) )
+  { gp = allocGlobal(6);
+    at = &gp[1];
+    setVar(*at);
+    gp[0] = consPtr(&gp[1], TAG_ATTVAR|STG_GLOBAL);
+    *p = makeRef(&gp[0]);
+  } else
+  { gp = allocGlobal(5);
+    at = &gp[0];
+    setVar(*at);
+    *p = consPtr(&gp[0], TAG_ATTVAR|STG_GLOBAL);
+  }
+
+  at[1] = FUNCTOR_att3;
+  at[2] = name;
+  at[3] = linkVal(value);
+  at[4] = ATOM_nil;
+  at[0] = consPtr(&at[1], TAG_COMPOUND|STG_GLOBAL);
+
+  Trail(p);
+  succeed;
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int find_attr(Word av, atom_t name, Word *vp)
 
@@ -441,14 +492,13 @@ PRED_IMPL("put_attr", 3, put_attr3, 0)	/* +Var, +Name, +Value */
     vp = p;
   }
 
-  requireStack(global, 4*sizeof(word));
+  requireStack(global, 6*sizeof(word));
 
   av = valTermRef(A1);
   deRef(av);
 
   if ( isVar(*av) )
-  { make_new_attvar(av PASS_LD);
-    return put_attr(av, name, vp PASS_LD);
+  { return put_new_attvar(av, name, vp PASS_LD);
   } else if ( isAttVar(*av) )
   { return put_attr(av, name, vp PASS_LD);
   } else
@@ -467,7 +517,7 @@ PRED_IMPL("put_attrs", 2, put_attrs, 0)
   deRef(av);
 
   if ( isVar(*av) )
-  { make_new_attvar(av PASS_LD);
+  { make_new_attvar(av PASS_LD);	/* TBD: Merge */
     deRef(av);
   } else if ( !isAttVar(*av) )
   { return PL_error("put_attrs", 2, NULL, ERR_MUST_BE_VAR, 1, A1);
@@ -545,8 +595,7 @@ PRED_IMPL("$freeze", 2, freeze, PL_FA_TRANSPARENT)
     gt[2] = *valTermRef(g);
 
     if ( isVar(*v) )
-    { make_new_attvar(v PASS_LD);		/* <= 2 cells global */
-      put_attr(v, ATOM_freeze, &goal PASS_LD);	/* <= 4 cells global */
+    { put_new_attvar(v, ATOM_freeze, &goal PASS_LD); /* <= 6 cells global */
     } else
     { Word vp;
 
