@@ -186,17 +186,7 @@ static const PL_extension foreigns[] = {
   FRG("clause",    		3, pl_clause3,	        NDET|META|CREF),
   FRG("$clause",	        4, pl_clause4,	        NDET|META|CREF),
   FRG("nth_clause", 		3, pl_nth_clause,       NDET|META|CREF),
-  FRG("retract",    		1, pl_retract,          NDET|META|CREF|ISO),
   FRG("retractall",		1, pl_retractall,	 META|ISO),
-#if O_DEBUGGER
-  FRG("$xr_member",		2, pl_xr_member,        NDET|META),
-  FRG("$wam_list",		1, pl_wam_list,			0),
-  FRG("$fetch_vm",		4, pl_fetch_vm,			0),
-  FRG("$clause_term_position",	3, pl_clause_term_position,	0),
-  FRG("$break_pc",		3, pl_break_pc,		     NDET),
-  FRG("$break_at",		3, pl_break_at,			0),
-  FRG("$current_break",		2, pl_current_break,	     NDET),
-#endif  
 #ifdef O_MAINTENANCE
   FRG("$list_generations",	1, pl_list_generations,	     META),
   FRG("$check_procedure",	1, pl_check_procedure,	     META),
@@ -262,7 +252,6 @@ static const PL_extension foreigns[] = {
   FRG("$declare_module",	3, pl_declare_module,		0),
   FRG("context_module",		1, pl_context_module,	     META),
   FRG("import",			1, pl_import,		     META),
-  FRG("$check_export",		0, pl_check_export,	     META),
   FRG("export_list",		2, pl_export_list,		0),
   FRG("index",			1, pl_index,		     META),
   FRG("hash",			1, pl_hash,		     META),
@@ -289,7 +278,6 @@ static const PL_extension foreigns[] = {
 #endif /* O_STRING */
 
   FRG("$length",		2, pl_length,			0),
-  FRG("memberchk",		2, pl_memberchk,		0),
   FRG("format",			2, pl_format,		     META),
 #ifdef O_DEBUG
   FRG("$check_definition",	1, pl_check_definition,      META),
@@ -521,7 +509,6 @@ DECL_PLIST(list);
 DECL_PLIST(module);
 DECL_PLIST(prims);
 DECL_PLIST(read);
-DECL_PLIST(wam);
 DECL_PLIST(thread);
 DECL_PLIST(profile);
 DECL_PLIST(wic);
@@ -551,7 +538,6 @@ initBuildIns(void)
   REG_PLIST(module);
   REG_PLIST(prims);
   REG_PLIST(read);
-  REG_PLIST(wam);
   REG_PLIST(thread);
   REG_PLIST(profile);
   REG_PLIST(wic);
