@@ -1329,6 +1329,17 @@ PL_get_int64(term_t t, int64_t *i)
 #define PL_get_int64(t, i) PL_get_int64__LD(t, i PASS_LD)
 
 
+int
+PL_get_intptr(term_t t, intptr_t *i)
+{ GET_LD
+#if SIZEOF_LONG != SIZEOF_VOIDP && SIZEOF_VOIDP == 8
+   return PL_get_int64(t, i);
+#else
+   return PL_get_long(t, (long*)i);
+#endif
+}
+
+
 int
 PL_is_inf(term_t t)
 { GET_LD
@@ -3165,8 +3176,12 @@ PL_foreign_control(control_t h)
 int
 PL_raise_exception(term_t exception)
 { GET_LD
-  PL_put_term(exception_bin, exception);
+
+  setVar(*valTermRef(exception_bin));
+  if ( !duplicate_term(exception, exception_bin PASS_LD) )
+    fatalError("Failed to copy exception term");
   exception_term = exception_bin;
+  freezeGlobal(PASS_LD1);
 
   fail;
 }
@@ -3176,10 +3191,7 @@ int
 PL_throw(term_t exception)
 { GET_LD
 
-  PL_put_term(exception_bin, exception);
-  exception_term = exception_bin;
-  assert(exception_term);
-
+  PL_raise_exception(exception);
   if ( LD->exception.throw_environment )
     longjmp(LD->exception.throw_environment->exception_jmp_env, 1);
 
