@@ -274,7 +274,7 @@ format_impl(IOSTREAM *out, term_t format, term_t Args)
   if ( !PL_get_text(format, &fmt, CVT_ALL|BUF_RING) )
     return PL_error("format", 3, NULL, ERR_TYPE, ATOM_text, format);
 
-  if ( (argc = lengthList(args, FALSE)) >= 0 )
+  if ( (argc = (int)lengthList(args, FALSE)) >= 0 )
   { term_t head = PL_new_term_ref();
     int n = 0;
 
@@ -674,9 +674,13 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv)
 		  if ( str != buf )
 		    free(str);
 
-		  if ( !rval && ex )
+		  if ( !rval )
 		  { Sunlock(fd);
-		    return PL_raise_exception(ex);
+
+		    if ( ex )
+		      return PL_raise_exception(ex);
+		    else
+		      fail;
 		  }
 
 		  SHIFT;
@@ -842,14 +846,21 @@ formatNumber(bool split, int div, int radix, bool small, Number i,
 { switch(i->type)
   { case V_INTEGER:
     { int64_t n = i->value.i;
-
-      char tmp[100]; 
-      char *end = &tmp[sizeof(tmp)];
-      char *s = end;			/* i.e. start at the end */
+      char buf[100]; 
+      char *tmp, *end, *s;
       int before = (div == 0);
       int digits = 0;
       bool negative = FALSE;
 
+      if ( div+3 > (int)sizeof(buf) )	/* 0.000NNNN with div digits after 0. */
+      { tmp = PL_malloc(div+3);
+	end = tmp+div+3;
+      } else
+      { tmp = buf;
+	end = tmp+sizeof(buf);
+      }
+
+      s = end;				/* i.e. start at the end */
       *--s = EOS;
       if ( n < 0 )
       { n = -n;
@@ -873,6 +884,8 @@ formatNumber(bool split, int div, int radix, bool small, Number i,
       }
 
       addMultipleBuffer(out, s, end-s, char);
+      if ( tmp != buf )
+	PL_free(tmp);
 
       return baseBuffer(out, char);
     }
