@@ -85,7 +85,7 @@ COMMON(void) 		do_undo(mark *m);
 COMMON(Definition) 	getProcDefinition(Procedure proc);
 COMMON(void) 		fix_term_ref_count(void);
 COMMON(fid_t) 		PL_open_signal_foreign_frame(void);
-COMMON(bool)		foreignWakeup(ARG1_LD);
+COMMON(bool)		foreignWakeup(term_t *ex ARG_LD);
 
 /* pl-atom.c */
 COMMON(word) 		lookupAtom(const char *s, size_t len);
@@ -142,8 +142,9 @@ COMMON(Clause) 		compileClause(Word head, Word body,
 			      Procedure proc, Module module ARG_LD);
 COMMON(Clause) 		assert_term(term_t term, int where,
 				    SourceLoc loc ARG_LD);
-COMMON(void) 		unregisterAtomsClause(Clause clause);
+COMMON(void) 		forAtomsInClause(Clause clause, void (func)(atom_t a));
 COMMON(word) 		pl_redefine_system_predicate(term_t term);
+COMMON(Code)		stepDynPC(Code PC, const code_info *ci);
 COMMON(bool) 		decompileHead(Clause clause, term_t head);
 COMMON(int) 		arg1Key(Clause clause, word *key);
 COMMON(bool) 		decompile(Clause clause, term_t term, term_t bindings);
@@ -154,23 +155,13 @@ COMMON(word) 		pl_clause3(term_t p, term_t term, term_t ref,
 COMMON(word) 		pl_clause2(term_t p, term_t term, control_t h);
 COMMON(word) 		pl_nth_clause(term_t p, term_t n, term_t ref,
 				      control_t h);
-COMMON(word) 		pl_xr_member(term_t ref, term_t term, control_t h);
 COMMON(void) 		wamListClause(Clause clause);
 COMMON(Code) 		wamListInstruction(IOSTREAM *out, Clause clause,
 					   Code bp);
-COMMON(word) 		pl_wam_list(term_t ref);
-COMMON(word) 		pl_fetch_vm(term_t ref, term_t offset, term_t noffset,
-				    term_t instruction);
 COMMON(int) 		unify_definition(term_t head, Definition def,
 				 term_t thehead, int flags);
-COMMON(word) 		pl_clause_term_position(term_t ref, term_t pc,
-						term_t locterm);
-COMMON(word) 		pl_break_pc(term_t ref, term_t pc, term_t nextpc,
-				    control_t h);
-COMMON(word) 		pl_break_at(term_t ref, term_t pc, term_t set);
 COMMON(code) 		replacedBreak(Code PC);
 COMMON(void) 		clearBreakPointsClause(Clause clause);
-COMMON(word) 		pl_current_break(term_t ref, term_t pc, control_t h);
 COMMON(int) 		get_clause_ptr_ex(term_t ref, Clause *cl);
 COMMON(int)		unify_functor(term_t t, functor_t fd, int how);
 
@@ -457,7 +448,6 @@ COMMON(void) 		resetForeign(void);
 
 /* pl-list.c */
 COMMON(word) 		pl_length(term_t list, term_t l);
-COMMON(word) 		pl_memberchk(term_t e, term_t list);
 
 /* pl-load.c */
 COMMON(word) 		pl_open_shared_object(term_t file, term_t h, term_t flags);
@@ -479,7 +469,6 @@ COMMON(word) 		pl_module(term_t old, term_t new);
 COMMON(word) 		pl_set_source_module(term_t old, term_t new);
 COMMON(word) 		pl_declare_module(term_t name, term_t file, term_t line);
 COMMON(word) 		pl_export_list(term_t modulename, term_t list);
-COMMON(word) 		pl_check_export(void);
 COMMON(word) 		pl_context_module(term_t module);
 COMMON(word) 		pl_import(term_t pred);
 #ifdef O_PROLOG_HOOK
@@ -545,11 +534,12 @@ COMMON(int) 		Pause(real time);
 /* pl-prims.c */
 COMMON(bool) 		raw_unify_ptrs(Word t1, Word t2 ARG_LD);
 COMMON(bool) 		unify_ptrs(Word t1, Word t2 ARG_LD);
-COMMON(bool) 		can_unify(Word t1, Word t2);
+COMMON(bool) 		can_unify(Word t1, Word t2, term_t *ex);
 COMMON(int) 		compareStandard(Word t1, Word t2, int eq ARG_LD);
 COMMON(intptr_t)	skip_list(Word l, Word *tailp ARG_LD);
 COMMON(intptr_t) 	lengthList(term_t list, int errors);
 COMMON(int) 		numberVars(term_t t, nv_options *opts, int n ARG_LD);
+COMMON(int)		duplicate_term(term_t in, term_t copy ARG_LD);
 COMMON(word) 		pl_e_free_variables(term_t t, term_t l);
 COMMON(word) 		stringToList(char *s);
 COMMON(word) 		pl_atom_concat(term_t a1, term_t a2, term_t a3,
@@ -625,7 +615,6 @@ COMMON(void) 		destroyDefinition(Definition def);
 COMMON(void) 		resetReferences(void);
 COMMON(Procedure) 	resolveProcedure(functor_t f, Module module);
 COMMON(Definition) 	trapUndefined(LocalFrame *frp, Code PC, Procedure proc ARG_LD);
-COMMON(word) 		pl_retract(term_t term, control_t h);
 COMMON(word) 		pl_retractall(term_t head);
 COMMON(word) 		pl_abolish(term_t atom, term_t arity);
 COMMON(word) 		pl_abolish1(term_t pred);
@@ -695,7 +684,6 @@ COMMON(word) 		pl_recordz(term_t key, term_t term, term_t ref);
 COMMON(word) 		pl_recorded(term_t key, term_t term, term_t ref, control_t h);
 COMMON(word) 		pl_erase(term_t ref);
 COMMON(word) 		pl_term_complexity(term_t t, term_t mx, term_t count);
-COMMON(void) 		undo_while_saving_term(mark *m, Word term);
 COMMON(void)		markAtomsRecord(Record record);
 
 /* pl-rl.c */
