@@ -45,7 +45,7 @@ typedef struct
 #if VMCODE_IS_ADDRESS
   char   *_dewam_table;			/* decoding table */
   intptr_t	  _dewam_table_offset;		/* offset of 1st */
-  code    _wam_table[I_HIGHEST+1];	/* code --> address */
+  code    _wam_table[I_HIGHEST];	/* code --> address */
   void  **_interpreter_jmp_table;	/* interpreters table */
 #else
   int	  struct_may_not_be_empty;	/* empty structure is illegal */
@@ -357,6 +357,10 @@ typedef struct PL_local_data
   } attvar;
 #endif
 
+  struct
+  { term_t	dummy;			/* see trimStacks() */
+  } trim;
+
 #ifdef O_GVAR
   struct
   { Table	nb_vars;		/* atom --> value */
