@@ -35,6 +35,9 @@ option  parsing,  initialisation  and  handling  of errors and warnings.
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
+#ifdef __WINDOWS__
+#include <process.h>			/* getpid() */
+#endif
 
 #define LOCK()   PL_LOCK(L_INIT)
 #define UNLOCK() PL_UNLOCK(L_INIT)
@@ -1341,8 +1344,14 @@ vsysError(const char *fm, va_list args)
     PL_halt(1);
 
 action:
-  Sfprintf(Serror, "\nAction? "); Sflush(Soutput);
+#ifdef HAVE_GETPID
+  Sfprintf(Serror, "\n[pid=%d] Action? ", getpid());
+#else
+  Sfprintf(Serror, "\nAction? ");
+#endif
+  Sflush(Soutput);
   ResetTty();
+
   switch(getSingleChar(Sinput, FALSE))
   { case 'a':
       pl_abort(ABORT_FATAL);
