@@ -1824,84 +1824,3 @@ pl_term_complexity(term_t t, term_t mx, term_t count)
 
   return PL_unify_integer(count, c);
 }
-
-
-		 /*******************************
-		 *     CATCH/THROW SUPPORT	*
-		 *******************************/
-
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-undo_while_saving_term(Mark m, Word term)
-
-Undo(m), but preserve `term'. Variables in  term, sharing with variables
-older then the mark should be remain sharing.
-
-The implementation exploits the  recorded   database  primitives defined
-above to perform a term-copy. It merges   the two routines. In addition,
-it uses the variable info gathered by the compiler and copier to restore
-the variable bindings. 
-
-This could also be used for a foreign implementation of findall.  Wonder
-whether that is worth the trouble?
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-void
-undo_while_saving_term(mark *m, Word term)
-{ GET_LD
-  compile_info info;
-  copy_info b;
-  uint n;
-  Word *p;
-
-  SECURE(checkData(term));
-  assert(onStack(local, term));
-
-  init_cycle(PASS_LD1);
-  initBuffer(&info.code);
-  initBuffer(&info.vars);
-  info.size = 0;
-  info.nvars = 0;
-  info.external = FALSE;
-  info.lock = FALSE;
-
-  compile_term_to_heap(term, &info PASS_LD);
-  p = topBuffer(&info.vars, Word);
-  while(p > baseBuffer(&info.vars, Word))
-  { p--;
-    if (isAttVarP(*p) )
-    { *valAttVarP(*p) = (word)p[-1];
-      p--;
-    } else
-      setVar(**p);
-  }
-
-  unvisit(PASS_LD1);
-  Undo(*m);
-  
-  b.data = info.code.base;
-  b.gbase = b.gstore = allocGlobal(info.size);
-  INITCOPYVARS(b, info.nvars);
-  copy_record(term, &b PASS_LD);
-  FREECOPYVARS(b, info.nvars);
-  assert(b.gstore == gTop);
-  discardBuffer(&info.code);
-
-  p = topBuffer(&info.vars, Word);
-  n = info.nvars;
-  while(p > baseBuffer(&info.vars, Word))
-  { p--;
-    n--;
-    if (isAttVarP(*p) )
-    { p--;				/* what to do? */
-    } else
-    { Word v = *p;
-
-      if ( onStack(local, v) || (v > gBase && v < m->globaltop) )
-	unify_ptrs(v, ((Word *)b.vars)[n] PASS_LD);
-    }
-  }
-
-  discardBuffer(&info.vars);
-}
-
-
