@@ -361,7 +361,7 @@ to the main thread.
 #define LD LOCAL_LD
 
 static void
-dispatch_signal(int sig, int sync)
+dispatch_signal(int sig, int sync, Code PC)
 { GET_LD
   SigHandler sh = &GD->sig_handlers[sig];
   fid_t fid;
@@ -400,7 +400,7 @@ dispatch_signal(int sig, int sync)
 	sysError("Recursively received fatal signal %d", sig);
   }
 
-  if ( gc_status.active )
+  if ( gc_status.active && sig < SIG_PROLOG_OFFSET )
   { fatalError("Received signal %d (%s) while in %ld-th garbage collection",
 	       sig, signal_name(sig), gc_status.collections);
   }
@@ -429,6 +429,8 @@ dispatch_signal(int sig, int sync)
 			PL_Q_CATCH_EXCEPTION,
 			sh->predicate,
 			sigterm);
+    if ( PC )
+      QueryFromQid(qid)->saved_PC = PC;
     if ( !PL_next_solution(qid) && (except = PL_exception(qid)) )
     { PL_cut_query(qid);
       if ( !sync )
@@ -486,7 +488,7 @@ dispatch_signal(int sig, int sync)
 
 static void
 pl_signal_handler(int sig)
-{ dispatch_signal(sig, FALSE);
+{ dispatch_signal(sig, FALSE, NULL);
 }
 
 #ifndef SA_RESTART
@@ -808,6 +810,17 @@ PL_signal(int sigandflags, handler_t func)
 
 int
 PL_handle_signals()
+{ PL_local_data_t *ld = LD;
+
+  if ( !ld || ld->critical || !ld->pending_signals )
+    return 0;
+
+  return handleSignals(NULL);
+}
+
+
+int
+handleSignals(Code PC)
 { int done = 0;
   PL_local_data_t *ld = LD;
 
@@ -823,7 +836,7 @@ PL_handle_signals()
       { ld->pending_signals &= ~mask;	/* reset the signal */
 
 	done++;
-	dispatch_signal(sig, TRUE);
+	dispatch_signal(sig, TRUE, PC);
 
 	if ( exception_term )
 	  goto out;
@@ -1028,6 +1041,7 @@ emptyStacks()
   exception_printed     = PL_new_term_ref();
   LD->exception.tmp     = PL_new_term_ref();
   LD->exception.pending = PL_new_term_ref();
+  LD->trim.dummy        = PL_new_term_ref();
 #ifdef O_ATTVAR
   LD->attvar.head	= PL_new_term_ref();
   LD->attvar.tail       = PL_new_term_ref();
@@ -1260,9 +1274,32 @@ unmap(Stack s)
 
 /* mmap() version */
 
+#ifdef SECURE_GC
+#include <stdio.h>
+
+static void
+fixed_bases(void **gbase, void**tbase)
+{ const char *bases;
+
+  if ( PL_thread_self() == 1 && (bases=getenv("PL_STACK_BASES")) )
+  { unsigned long gb, tb;
+
+    if ( sscanf(bases, "G%lx+T%lx", &gb, &tb) == 2 )
+    { *gbase = (void*)gb;
+      *tbase = (void*)tb;
+    } else
+    { Sdprintf("Usage: export PL_STACK_BASES=\"G<gBase>+T<tBase>\"\n");
+      exit(1);
+    }
+  }
+}
+
+#endif /*SECURE_GC*/
+
 static int
 allocStacks(intptr_t local, intptr_t global, intptr_t trail, intptr_t argument)
-{ caddress lbase, gbase, tbase, abase;
+{ caddress lbase, gbase=NULL, tbase=NULL, abase=NULL;
+  int flags = MAP_FLAGS;
   intptr_t glsize;
   intptr_t minglobal   = 4*SIZEOF_VOIDP K;
   intptr_t minlocal    = 2*SIZEOF_VOIDP K;
@@ -1291,9 +1328,18 @@ allocStacks(intptr_t local, intptr_t global, intptr_t trail, intptr_t argument)
   argument = (intptr_t) align_size(argument);
   glsize   = global+local;
 
-  tbase = mmap(NULL, trail,    MAPPROTECT, MAP_FLAGS, mapfd, 0L);
-  abase = mmap(NULL, argument, MAPPROTECT, MAP_FLAGS, mapfd, 0L);
-  gbase = mmap(NULL, glsize,   MAPPROTECT, MAP_FLAGS, mapfd, 0L);
+#ifdef SECURE_GC
+  fixed_bases(&gbase, &tbase);
+  if ( gbase || tbase )
+  { flags |= MAP_FIXED;
+    Sdprintf("Using fixed stack bases at gBase=0x%lx; tBase=0x%lx\n",
+	     gbase, tbase);
+  }
+#endif
+
+  tbase = mmap(tbase, trail,    MAPPROTECT, flags, mapfd, 0L);
+  abase = mmap(abase, argument, MAPPROTECT, MAP_FLAGS, mapfd, 0L);
+  gbase = mmap(gbase, glsize,   MAPPROTECT, flags, mapfd, 0L);
   lbase = addPointer(gbase, global);
 
   if ( tbase == MAP_FAILED || abase == MAP_FAILED || gbase == MAP_FAILED )
@@ -1307,6 +1353,12 @@ allocStacks(intptr_t local, intptr_t global, intptr_t trail, intptr_t argument)
     fail;
   }
 
+#ifdef SECURE_GC
+  if ( !(flags & MAP_FIXED) && PL_thread_self() == 1 )
+    Sdprintf("Realised export PL_STACK_BASES=\"G0x%lx+T0x%lx\"\n",
+	     (unsigned long)gbase, (unsigned long)tbase);
+#endif
+
 #define INIT_STACK(name, print, base, limit, minsize) \
   DEBUG(1, Sdprintf("%s stack at 0x%x; size = %ld\n", print, base, limit)); \
   init_stack((Stack) &LD->stacks.name, print, base, limit, minsize);
@@ -1769,7 +1821,7 @@ trimStacks() reclaims all unused space on the stack. Note that the trail
 can have references to unused stack. We set the references to point to a
 dummy variable, so no harm  will  be   done.  Setting  it  to NULL would
 require a test in Undo(), which   is time-critical. trim_stacks normally
-isn't. This precaution is explicitly requires  for the trimStacks() that
+isn't. This precaution is explicitly required  for the trimStacks() that
 result from a stack-overflow.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
@@ -1779,7 +1831,6 @@ trimStacks(ARG1_LD)
 #undef LD
 #define LD LOCAL_LD
   TrailEntry te;
-  Word dummy = NULL;
 
   LD->trim_stack_requested = FALSE;
 
@@ -1795,6 +1846,16 @@ trimStacks(ARG1_LD)
 #endif /*O_DYNAMIC_STACKS*/
 #endif /*O_SHIFT_STACKS*/
 
+#ifdef SECURE_GC
+  { Word p;				/* clear the stacks */
+
+    for(p=gTop; p<gMax; p++)
+      *p = 0xbfbfbfbf;
+    for(p=(Word)lTop; p<(Word)lMax; p++)
+      *p = 0xbfbfbfbf;
+  }
+#endif
+
   for(te = tTop; --te >= tBase; )
   { Word p = te->address;
     
@@ -1802,12 +1863,7 @@ trimStacks(ARG1_LD)
       continue;
 
     if ( !onStack(local, p) && !onStack(global, p) )
-    { if ( !dummy )
-      { dummy = allocGlobal(1);
-	setVar(*dummy);
-      }
-
-      te->address = dummy;
+    { te->address = valTermRef(LD->trim.dummy);
     }
   }
 #undef LD
