@@ -214,6 +214,7 @@ canUnifyTermWithGoal(LocalFrame fr)
 	Word a, b;
 	int arity = fr->predicate->functor->arity;
 	int rval = TRUE;
+	term_t ex;
 
 	copyRecordToGlobal(t, find->goal.term.term PASS_LD);
 	a = valTermRef(t);
@@ -221,7 +222,7 @@ canUnifyTermWithGoal(LocalFrame fr)
 	a = argTermP(*a, 0);
 	b = argFrameP(fr, 0);
 	while( arity-- > 0 )
-	{ if ( !can_unify(a++, b++) )
+	{ if ( !can_unify(a++, b++, &ex) )
 	  { rval = FALSE;
 	    break;
 	  }
@@ -1554,6 +1555,8 @@ pl_prolog_frame_attribute(term_t frame, term_t what,
 
   if ( !PL_get_frame(frame, &fr) )
     return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_frame_reference, frame);
+  if ( !fr )
+    fail;				/* frame == 'none' */
   if ( !PL_get_name_arity(what, &key, &arity) )
     return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_callable, what);
 
@@ -1694,6 +1697,7 @@ pl_prolog_frame_attribute(term_t frame, term_t what,
   { if ( fr->programPointer &&
 	 fr->parent &&
 	 false(fr->parent->predicate, FOREIGN) &&
+	 fr->parent->clause &&
 	 fr->parent->predicate != PROCEDURE_dcall1->definition )
     { intptr_t pc = fr->programPointer - fr->parent->clause->clause->codes;
 
