@@ -1772,7 +1772,7 @@ avar(wakeup-7) :-
 	test_wakeup(functor).
 avar(type-1) :-
 	put_attr(X, test, a),
-	avar(X).
+	attvar(X).
 avar(type-2) :-
 	put_attr(X, test, a),
 	var(X).
@@ -2390,6 +2390,8 @@ th_do_something :-
 th_check_done :-
 	findall(X, retract(th_data(X)), [1,2,3,4,5]).
 
+thread(queue-1) :-			% crash if no queue exists?
+	findall(Q, message_queue_property(Q,_), _).
 thread(join-1) :-
 	thread_create(th_do_something, Id, []),
 	thread_join(Id, Exit),
@@ -2418,6 +2420,9 @@ thread(status-1) :-
 	Id2 == Id,
 	Status == true, !,
 	thread_join(Id2, _).
+thread(create_error-1) :-
+	catch(thread_create(true, _, [local(a)]), E, true),
+	E = error(domain_error(thread_option, local(a)), _).
 	
 
 		 /*******************************
